"""
Python module generated from Java source file org.yaml.snakeyaml.emitter.EmitterState

Java source file obtained from artifact snakeyaml version 2.2

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import IOException
from org.yaml.snakeyaml.emitter import *
from typing import Any, Callable, Iterable, Tuple


class EmitterState:
    """
    Python's methods are first class object. Java needs a class.
    """

    def expect(self) -> None:
        ...



"""
Python module generated from Java source file java.util.Deque

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import *
from typing import Any, Callable, Iterable, Tuple


class Deque(Queue):
    """
    A linear collection that supports element insertion and removal at
    both ends.  The name *deque* is short for "double ended queue"
    and is usually pronounced "deck".  Most `Deque`
    implementations place no fixed limits on the number of elements
    they may contain, but this interface supports capacity-restricted
    deques as well as those with no fixed size limit.
    
    This interface defines methods to access the elements at both
    ends of the deque.  Methods are provided to insert, remove, and
    examine the element.  Each of these methods exists in two forms:
    one throws an exception if the operation fails, the other returns a
    special value (either `null` or `False`, depending on
    the operation).  The latter form of the insert operation is
    designed specifically for use with capacity-restricted
    `Deque` implementations; in most implementations, insert
    operations cannot fail.
    
    The twelve methods described above are summarized in the
    following table:
    
    <table class="striped">
    <caption>Summary of Deque methods</caption>
     <thead>
     <tr>
       <td rowspan="2"></td>
       <th scope="col" colspan="2"> First Element (Head)</th>
       <th scope="col" colspan="2"> Last Element (Tail)</th>
     </tr>
     <tr>
       <th scope="col" style="font-weight:normal; font-style:italic">Throws exception</th>
       <th scope="col" style="font-weight:normal; font-style:italic">Special value</th>
       <th scope="col" style="font-weight:normal; font-style:italic">Throws exception</th>
       <th scope="col" style="font-weight:normal; font-style:italic">Special value</th>
     </tr>
     </thead>
     <tbody>
     <tr>
       <th scope="row">Insert</th>
       <td>.addFirst(Object) addFirst(e)</td>
       <td>.offerFirst(Object) offerFirst(e)</td>
       <td>.addLast(Object) addLast(e)</td>
       <td>.offerLast(Object) offerLast(e)</td>
     </tr>
     <tr>
       <th scope="row">Remove</th>
       <td>.removeFirst() removeFirst()</td>
       <td>.pollFirst() pollFirst()</td>
       <td>.removeLast() removeLast()</td>
       <td>.pollLast() pollLast()</td>
     </tr>
     <tr>
       <th scope="row">Examine</th>
       <td>.getFirst() getFirst()</td>
       <td>.peekFirst() peekFirst()</td>
       <td>.getLast() getLast()</td>
       <td>.peekLast() peekLast()</td>
     </tr>
     </tbody>
    </table>
    
    This interface extends the Queue interface.  When a deque is
    used as a queue, FIFO (First-In-First-Out) behavior results.  Elements are
    added at the end of the deque and removed from the beginning.  The methods
    inherited from the `Queue` interface are precisely equivalent to
    `Deque` methods as indicated in the following table:
    
    <table class="striped">
    <caption>Comparison of Queue and Deque methods</caption>
     <thead>
     <tr>
       <th scope="col"> `Queue` Method</th>
       <th scope="col"> Equivalent `Deque` Method</th>
     </tr>
     </thead>
     <tbody>
     <tr>
       <th scope="row">.add(Object) add(e)</th>
       <td>.addLast(Object) addLast(e)</td>
     </tr>
     <tr>
       <th scope="row">.offer(Object) offer(e)</th>
       <td>.offerLast(Object) offerLast(e)</td>
     </tr>
     <tr>
       <th scope="row">.remove() remove()</th>
       <td>.removeFirst() removeFirst()</td>
     </tr>
     <tr>
       <th scope="row">.poll() poll()</th>
       <td>.pollFirst() pollFirst()</td>
     </tr>
     <tr>
       <th scope="row">.element() element()</th>
       <td>.getFirst() getFirst()</td>
     </tr>
     <tr>
       <th scope="row">.peek() peek()</th>
       <td>.peekFirst() peekFirst()</td>
     </tr>
     </tbody>
    </table>
    
    Deques can also be used as LIFO (Last-In-First-Out) stacks.  This
    interface should be used in preference to the legacy Stack class.
    When a deque is used as a stack, elements are pushed and popped from the
    beginning of the deque.  Stack methods are equivalent to `Deque`
    methods as indicated in the table below:
    
    <table class="striped">
    <caption>Comparison of Stack and Deque methods</caption>
     <thead>
     <tr>
       <th scope="col"> Stack Method</th>
       <th scope="col"> Equivalent `Deque` Method</th>
     </tr>
     </thead>
     <tbody>
     <tr>
       <th scope="row">.push(Object) push(e)</th>
       <td>.addFirst(Object) addFirst(e)</td>
     </tr>
     <tr>
       <th scope="row">.pop() pop()</th>
       <td>.removeFirst() removeFirst()</td>
     </tr>
     <tr>
       <th scope="row">.peek() peek()</th>
       <td>.getFirst() getFirst()</td>
     </tr>
     </tbody>
    </table>
    
    Note that the .peek peek method works equally well when
    a deque is used as a queue or a stack; in either case, elements are
    drawn from the beginning of the deque.
    
    This interface provides two methods to remove interior
    elements, .removeFirstOccurrence removeFirstOccurrence and
    .removeLastOccurrence removeLastOccurrence.
    
    Unlike the List interface, this interface does not
    provide support for indexed access to elements.
    
    While `Deque` implementations are not strictly required
    to prohibit the insertion of null elements, they are strongly
    encouraged to do so.  Users of any `Deque` implementations
    that do allow null elements are strongly encouraged *not* to
    take advantage of the ability to insert nulls.  This is so because
    `null` is used as a special return value by various methods
    to indicate that the deque is empty.
    
    `Deque` implementations generally do not define
    element-based versions of the `equals` and `hashCode`
    methods, but instead inherit the identity-based versions from class
    `Object`.
    
    This interface is a member of the
    <a href="/java.base/java/util/package-summary.html#CollectionsFramework">
    Java Collections Framework</a>.
    
    Type `<E>`: the type of elements held in this deque

    Author(s)
    - Josh Bloch

    Since
    - 1.6
    """

    def addFirst(self, e: "E") -> None:
        """
        Inserts the specified element at the front of this deque if it is
        possible to do so immediately without violating capacity restrictions,
        throwing an `IllegalStateException` if no space is currently
        available.  When using a capacity-restricted deque, it is generally
        preferable to use method .offerFirst.

        Arguments
        - e: the element to add

        Raises
        - IllegalStateException: if the element cannot be added at this
                time due to capacity restrictions
        - ClassCastException: if the class of the specified element
                prevents it from being added to this deque
        - NullPointerException: if the specified element is null and this
                deque does not permit null elements
        - IllegalArgumentException: if some property of the specified
                element prevents it from being added to this deque
        """
        ...


    def addLast(self, e: "E") -> None:
        """
        Inserts the specified element at the end of this deque if it is
        possible to do so immediately without violating capacity restrictions,
        throwing an `IllegalStateException` if no space is currently
        available.  When using a capacity-restricted deque, it is generally
        preferable to use method .offerLast.
        
        This method is equivalent to .add.

        Arguments
        - e: the element to add

        Raises
        - IllegalStateException: if the element cannot be added at this
                time due to capacity restrictions
        - ClassCastException: if the class of the specified element
                prevents it from being added to this deque
        - NullPointerException: if the specified element is null and this
                deque does not permit null elements
        - IllegalArgumentException: if some property of the specified
                element prevents it from being added to this deque
        """
        ...


    def offerFirst(self, e: "E") -> bool:
        """
        Inserts the specified element at the front of this deque unless it would
        violate capacity restrictions.  When using a capacity-restricted deque,
        this method is generally preferable to the .addFirst method,
        which can fail to insert an element only by throwing an exception.

        Arguments
        - e: the element to add

        Returns
        - `True` if the element was added to this deque, else
                `False`

        Raises
        - ClassCastException: if the class of the specified element
                prevents it from being added to this deque
        - NullPointerException: if the specified element is null and this
                deque does not permit null elements
        - IllegalArgumentException: if some property of the specified
                element prevents it from being added to this deque
        """
        ...


    def offerLast(self, e: "E") -> bool:
        """
        Inserts the specified element at the end of this deque unless it would
        violate capacity restrictions.  When using a capacity-restricted deque,
        this method is generally preferable to the .addLast method,
        which can fail to insert an element only by throwing an exception.

        Arguments
        - e: the element to add

        Returns
        - `True` if the element was added to this deque, else
                `False`

        Raises
        - ClassCastException: if the class of the specified element
                prevents it from being added to this deque
        - NullPointerException: if the specified element is null and this
                deque does not permit null elements
        - IllegalArgumentException: if some property of the specified
                element prevents it from being added to this deque
        """
        ...


    def removeFirst(self) -> "E":
        """
        Retrieves and removes the first element of this deque.  This method
        differs from .pollFirst pollFirst only in that it throws an
        exception if this deque is empty.

        Returns
        - the head of this deque

        Raises
        - NoSuchElementException: if this deque is empty
        """
        ...


    def removeLast(self) -> "E":
        """
        Retrieves and removes the last element of this deque.  This method
        differs from .pollLast pollLast only in that it throws an
        exception if this deque is empty.

        Returns
        - the tail of this deque

        Raises
        - NoSuchElementException: if this deque is empty
        """
        ...


    def pollFirst(self) -> "E":
        """
        Retrieves and removes the first element of this deque,
        or returns `null` if this deque is empty.

        Returns
        - the head of this deque, or `null` if this deque is empty
        """
        ...


    def pollLast(self) -> "E":
        """
        Retrieves and removes the last element of this deque,
        or returns `null` if this deque is empty.

        Returns
        - the tail of this deque, or `null` if this deque is empty
        """
        ...


    def getFirst(self) -> "E":
        """
        Retrieves, but does not remove, the first element of this deque.
        
        This method differs from .peekFirst peekFirst only in that it
        throws an exception if this deque is empty.

        Returns
        - the head of this deque

        Raises
        - NoSuchElementException: if this deque is empty
        """
        ...


    def getLast(self) -> "E":
        """
        Retrieves, but does not remove, the last element of this deque.
        This method differs from .peekLast peekLast only in that it
        throws an exception if this deque is empty.

        Returns
        - the tail of this deque

        Raises
        - NoSuchElementException: if this deque is empty
        """
        ...


    def peekFirst(self) -> "E":
        """
        Retrieves, but does not remove, the first element of this deque,
        or returns `null` if this deque is empty.

        Returns
        - the head of this deque, or `null` if this deque is empty
        """
        ...


    def peekLast(self) -> "E":
        """
        Retrieves, but does not remove, the last element of this deque,
        or returns `null` if this deque is empty.

        Returns
        - the tail of this deque, or `null` if this deque is empty
        """
        ...


    def removeFirstOccurrence(self, o: "Object") -> bool:
        """
        Removes the first occurrence of the specified element from this deque.
        If the deque does not contain the element, it is unchanged.
        More formally, removes the first element `e` such that
        `Objects.equals(o, e)` (if such an element exists).
        Returns `True` if this deque contained the specified element
        (or equivalently, if this deque changed as a result of the call).

        Arguments
        - o: element to be removed from this deque, if present

        Returns
        - `True` if an element was removed as a result of this call

        Raises
        - ClassCastException: if the class of the specified element
                is incompatible with this deque
        (<a href="/java.base/java/util/Collection.html#optional-restrictions">optional</a>)
        - NullPointerException: if the specified element is null and this
                deque does not permit null elements
        (<a href="/java.base/java/util/Collection.html#optional-restrictions">optional</a>)
        """
        ...


    def removeLastOccurrence(self, o: "Object") -> bool:
        """
        Removes the last occurrence of the specified element from this deque.
        If the deque does not contain the element, it is unchanged.
        More formally, removes the last element `e` such that
        `Objects.equals(o, e)` (if such an element exists).
        Returns `True` if this deque contained the specified element
        (or equivalently, if this deque changed as a result of the call).

        Arguments
        - o: element to be removed from this deque, if present

        Returns
        - `True` if an element was removed as a result of this call

        Raises
        - ClassCastException: if the class of the specified element
                is incompatible with this deque
        (<a href="/java.base/java/util/Collection.html#optional-restrictions">optional</a>)
        - NullPointerException: if the specified element is null and this
                deque does not permit null elements
        (<a href="/java.base/java/util/Collection.html#optional-restrictions">optional</a>)
        """
        ...


    def add(self, e: "E") -> bool:
        """
        Inserts the specified element into the queue represented by this deque
        (in other words, at the tail of this deque) if it is possible to do so
        immediately without violating capacity restrictions, returning
        `True` upon success and throwing an
        `IllegalStateException` if no space is currently available.
        When using a capacity-restricted deque, it is generally preferable to
        use .offer(Object) offer.
        
        This method is equivalent to .addLast.

        Arguments
        - e: the element to add

        Returns
        - `True` (as specified by Collection.add)

        Raises
        - IllegalStateException: if the element cannot be added at this
                time due to capacity restrictions
        - ClassCastException: if the class of the specified element
                prevents it from being added to this deque
        - NullPointerException: if the specified element is null and this
                deque does not permit null elements
        - IllegalArgumentException: if some property of the specified
                element prevents it from being added to this deque
        """
        ...


    def offer(self, e: "E") -> bool:
        """
        Inserts the specified element into the queue represented by this deque
        (in other words, at the tail of this deque) if it is possible to do so
        immediately without violating capacity restrictions, returning
        `True` upon success and `False` if no space is currently
        available.  When using a capacity-restricted deque, this method is
        generally preferable to the .add method, which can fail to
        insert an element only by throwing an exception.
        
        This method is equivalent to .offerLast.

        Arguments
        - e: the element to add

        Returns
        - `True` if the element was added to this deque, else
                `False`

        Raises
        - ClassCastException: if the class of the specified element
                prevents it from being added to this deque
        - NullPointerException: if the specified element is null and this
                deque does not permit null elements
        - IllegalArgumentException: if some property of the specified
                element prevents it from being added to this deque
        """
        ...


    def remove(self) -> "E":
        """
        Retrieves and removes the head of the queue represented by this deque
        (in other words, the first element of this deque).
        This method differs from .poll() poll() only in that it
        throws an exception if this deque is empty.
        
        This method is equivalent to .removeFirst().

        Returns
        - the head of the queue represented by this deque

        Raises
        - NoSuchElementException: if this deque is empty
        """
        ...


    def poll(self) -> "E":
        """
        Retrieves and removes the head of the queue represented by this deque
        (in other words, the first element of this deque), or returns
        `null` if this deque is empty.
        
        This method is equivalent to .pollFirst().

        Returns
        - the first element of this deque, or `null` if
                this deque is empty
        """
        ...


    def element(self) -> "E":
        """
        Retrieves, but does not remove, the head of the queue represented by
        this deque (in other words, the first element of this deque).
        This method differs from .peek peek only in that it throws an
        exception if this deque is empty.
        
        This method is equivalent to .getFirst().

        Returns
        - the head of the queue represented by this deque

        Raises
        - NoSuchElementException: if this deque is empty
        """
        ...


    def peek(self) -> "E":
        """
        Retrieves, but does not remove, the head of the queue represented by
        this deque (in other words, the first element of this deque), or
        returns `null` if this deque is empty.
        
        This method is equivalent to .peekFirst().

        Returns
        - the head of the queue represented by this deque, or
                `null` if this deque is empty
        """
        ...


    def addAll(self, c: Iterable["E"]) -> bool:
        """
        Adds all of the elements in the specified collection at the end
        of this deque, as if by calling .addLast on each one,
        in the order that they are returned by the collection's iterator.
        
        When using a capacity-restricted deque, it is generally preferable
        to call .offer(Object) offer separately on each element.
        
        An exception encountered while trying to add an element may result
        in only some of the elements having been successfully added when
        the associated exception is thrown.

        Arguments
        - c: the elements to be inserted into this deque

        Returns
        - `True` if this deque changed as a result of the call

        Raises
        - IllegalStateException: if not all the elements can be added at
                this time due to insertion restrictions
        - ClassCastException: if the class of an element of the specified
                collection prevents it from being added to this deque
        - NullPointerException: if the specified collection contains a
                null element and this deque does not permit null elements,
                or if the specified collection is null
        - IllegalArgumentException: if some property of an element of the
                specified collection prevents it from being added to this deque
        """
        ...


    def push(self, e: "E") -> None:
        """
        Pushes an element onto the stack represented by this deque (in other
        words, at the head of this deque) if it is possible to do so
        immediately without violating capacity restrictions, throwing an
        `IllegalStateException` if no space is currently available.
        
        This method is equivalent to .addFirst.

        Arguments
        - e: the element to push

        Raises
        - IllegalStateException: if the element cannot be added at this
                time due to capacity restrictions
        - ClassCastException: if the class of the specified element
                prevents it from being added to this deque
        - NullPointerException: if the specified element is null and this
                deque does not permit null elements
        - IllegalArgumentException: if some property of the specified
                element prevents it from being added to this deque
        """
        ...


    def pop(self) -> "E":
        """
        Pops an element from the stack represented by this deque.  In other
        words, removes and returns the first element of this deque.
        
        This method is equivalent to .removeFirst().

        Returns
        - the element at the front of this deque (which is the top
                of the stack represented by this deque)

        Raises
        - NoSuchElementException: if this deque is empty
        """
        ...


    def remove(self, o: "Object") -> bool:
        """
        Removes the first occurrence of the specified element from this deque.
        If the deque does not contain the element, it is unchanged.
        More formally, removes the first element `e` such that
        `Objects.equals(o, e)` (if such an element exists).
        Returns `True` if this deque contained the specified element
        (or equivalently, if this deque changed as a result of the call).
        
        This method is equivalent to .removeFirstOccurrence(Object).

        Arguments
        - o: element to be removed from this deque, if present

        Returns
        - `True` if an element was removed as a result of this call

        Raises
        - ClassCastException: if the class of the specified element
                is incompatible with this deque
        (<a href="/java.base/java/util/Collection.html#optional-restrictions">optional</a>)
        - NullPointerException: if the specified element is null and this
                deque does not permit null elements
        (<a href="/java.base/java/util/Collection.html#optional-restrictions">optional</a>)
        """
        ...


    def contains(self, o: "Object") -> bool:
        """
        Returns `True` if this deque contains the specified element.
        More formally, returns `True` if and only if this deque contains
        at least one element `e` such that `Objects.equals(o, e)`.

        Arguments
        - o: element whose presence in this deque is to be tested

        Returns
        - `True` if this deque contains the specified element

        Raises
        - ClassCastException: if the class of the specified element
                is incompatible with this deque
        (<a href="/java.base/java/util/Collection.html#optional-restrictions">optional</a>)
        - NullPointerException: if the specified element is null and this
                deque does not permit null elements
        (<a href="/java.base/java/util/Collection.html#optional-restrictions">optional</a>)
        """
        ...


    def size(self) -> int:
        """
        Returns the number of elements in this deque.

        Returns
        - the number of elements in this deque
        """
        ...


    def iterator(self) -> Iterator["E"]:
        """
        Returns an iterator over the elements in this deque in proper sequence.
        The elements will be returned in order from first (head) to last (tail).

        Returns
        - an iterator over the elements in this deque in proper sequence
        """
        ...


    def descendingIterator(self) -> Iterator["E"]:
        """
        Returns an iterator over the elements in this deque in reverse
        sequential order.  The elements will be returned in order from
        last (tail) to first (head).

        Returns
        - an iterator over the elements in this deque in reverse
        sequence
        """
        ...



"""
Python module generated from Java source file org.bukkit.entity.Item

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import UUID
from org.bukkit.entity import *
from org.bukkit.inventory import ItemStack
from typing import Any, Callable, Iterable, Tuple


class Item(Entity):
    """
    Represents a dropped item.
    """

    def getItemStack(self) -> "ItemStack":
        """
        Gets the item stack associated with this item drop.

        Returns
        - An item stack.
        """
        ...


    def setItemStack(self, stack: "ItemStack") -> None:
        """
        Sets the item stack associated with this item drop.

        Arguments
        - stack: An item stack.
        """
        ...


    def getPickupDelay(self) -> int:
        """
        Gets the delay before this Item is available to be picked up by players

        Returns
        - Remaining delay
        """
        ...


    def setPickupDelay(self, delay: int) -> None:
        """
        Sets the delay before this Item is available to be picked up by players

        Arguments
        - delay: New delay
        """
        ...


    def setUnlimitedLifetime(self, unlimited: bool) -> None:
        """
        Sets if this Item should live forever

        Arguments
        - unlimited: True if the lifetime is unlimited
        """
        ...


    def isUnlimitedLifetime(self) -> bool:
        """
        Gets if this Item lives forever

        Returns
        - True if the lifetime is unlimited
        """
        ...


    def setOwner(self, owner: "UUID") -> None:
        """
        Sets the owner of this item.
        
        Other entities will not be able to pickup this item when an owner is set.

        Arguments
        - owner: UUID of new owner
        """
        ...


    def getOwner(self) -> "UUID":
        """
        Get the owner of this item.

        Returns
        - UUID of owner
        """
        ...


    def setThrower(self, uuid: "UUID") -> None:
        """
        Set the thrower of this item.
        
        The thrower is the entity which dropped the item. This affects the
        trigger criteria for item pickups, for things such as advancements.

        Arguments
        - uuid: UUID of thrower
        """
        ...


    def getThrower(self) -> "UUID":
        """
        Get the thrower of this item.
        
        The thrower is the entity which dropped the item.

        Returns
        - UUID of thrower
        """
        ...



"""
Python module generated from Java source file org.bukkit.entity.Zoglin

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import *
from typing import Any, Callable, Iterable, Tuple


class Zoglin(Monster, Ageable):
    """
    Represents a Zoglin.
    """

    def isBaby(self) -> bool:
        """
        Gets whether the zoglin is a baby

        Returns
        - Whether the zoglin is a baby

        Deprecated
        - see Ageable.isAdult()
        """
        ...


    def setBaby(self, flag: bool) -> None:
        """
        Sets whether the zoglin is a baby

        Arguments
        - flag: Whether the zoglin is a baby

        Deprecated
        - see Ageable.setBaby() and Ageable.setAdult()
        """
        ...



"""
Python module generated from Java source file java.util.concurrent.CopyOnWriteArrayList

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.lang.invoke import VarHandle
from java.lang.reflect import Field
from java.util import Arrays
from java.util import Comparator
from java.util import ConcurrentModificationException
from java.util import Iterator
from java.util import ListIterator
from java.util import NoSuchElementException
from java.util import Objects
from java.util import RandomAccess
from java.util import Spliterator
from java.util.concurrent import *
from java.util.function import Consumer
from java.util.function import Predicate
from java.util.function import UnaryOperator
from jdk.internal.access import SharedSecrets
from typing import Any, Callable, Iterable, Tuple


class CopyOnWriteArrayList(List, RandomAccess, Cloneable, Serializable):
    """
    A thread-safe variant of java.util.ArrayList in which all mutative
    operations (`add`, `set`, and so on) are implemented by
    making a fresh copy of the underlying array.
    
    This is ordinarily too costly, but may be *more* efficient
    than alternatives when traversal operations vastly outnumber
    mutations, and is useful when you cannot or don't want to
    synchronize traversals, yet need to preclude interference among
    concurrent threads.  The "snapshot" style iterator method uses a
    reference to the state of the array at the point that the iterator
    was created. This array never changes during the lifetime of the
    iterator, so interference is impossible and the iterator is
    guaranteed not to throw `ConcurrentModificationException`.
    The iterator will not reflect additions, removals, or changes to
    the list since the iterator was created.  Element-changing
    operations on iterators themselves (`remove`, `set`, and
    `add`) are not supported. These methods throw
    `UnsupportedOperationException`.
    
    All elements are permitted, including `null`.
    
    Memory consistency effects: As with other concurrent
    collections, actions in a thread prior to placing an object into a
    `CopyOnWriteArrayList`
    <a href="package-summary.html#MemoryVisibility">*happen-before*</a>
    actions subsequent to the access or removal of that element from
    the `CopyOnWriteArrayList` in another thread.
    
    This class is a member of the
    <a href="/java.base/java/util/package-summary.html#CollectionsFramework">
    Java Collections Framework</a>.
    
    Type `<E>`: the type of elements held in this list

    Author(s)
    - Doug Lea

    Since
    - 1.5
    """

    def __init__(self):
        """
        Creates an empty list.
        """
        ...


    def __init__(self, c: Iterable["E"]):
        """
        Creates a list containing the elements of the specified
        collection, in the order they are returned by the collection's
        iterator.

        Arguments
        - c: the collection of initially held elements

        Raises
        - NullPointerException: if the specified collection is null
        """
        ...


    def __init__(self, toCopyIn: list["E"]):
        """
        Creates a list holding a copy of the given array.

        Arguments
        - toCopyIn: the array (a copy of this array is used as the
               internal array)

        Raises
        - NullPointerException: if the specified array is null
        """
        ...


    def size(self) -> int:
        """
        Returns the number of elements in this list.

        Returns
        - the number of elements in this list
        """
        ...


    def isEmpty(self) -> bool:
        """
        Returns `True` if this list contains no elements.

        Returns
        - `True` if this list contains no elements
        """
        ...


    def contains(self, o: "Object") -> bool:
        """
        Returns `True` if this list contains the specified element.
        More formally, returns `True` if and only if this list contains
        at least one element `e` such that `Objects.equals(o, e)`.

        Arguments
        - o: element whose presence in this list is to be tested

        Returns
        - `True` if this list contains the specified element
        """
        ...


    def indexOf(self, o: "Object") -> int:
        """

        """
        ...


    def indexOf(self, e: "E", index: int) -> int:
        """
        Returns the index of the first occurrence of the specified element in
        this list, searching forwards from `index`, or returns -1 if
        the element is not found.
        More formally, returns the lowest index `i` such that
        `i >= index && Objects.equals(get(i), e)`,
        or -1 if there is no such index.

        Arguments
        - e: element to search for
        - index: index to start searching from

        Returns
        - the index of the first occurrence of the element in
                this list at position `index` or later in the list;
                `-1` if the element is not found.

        Raises
        - IndexOutOfBoundsException: if the specified index is negative
        """
        ...


    def lastIndexOf(self, o: "Object") -> int:
        """

        """
        ...


    def lastIndexOf(self, e: "E", index: int) -> int:
        """
        Returns the index of the last occurrence of the specified element in
        this list, searching backwards from `index`, or returns -1 if
        the element is not found.
        More formally, returns the highest index `i` such that
        `i <= index && Objects.equals(get(i), e)`,
        or -1 if there is no such index.

        Arguments
        - e: element to search for
        - index: index to start searching backwards from

        Returns
        - the index of the last occurrence of the element at position
                less than or equal to `index` in this list;
                -1 if the element is not found.

        Raises
        - IndexOutOfBoundsException: if the specified index is greater
                than or equal to the current size of this list
        """
        ...


    def clone(self) -> "Object":
        """
        Returns a shallow copy of this list.  (The elements themselves
        are not copied.)

        Returns
        - a clone of this list
        """
        ...


    def toArray(self) -> list["Object"]:
        """
        Returns an array containing all of the elements in this list
        in proper sequence (from first to last element).
        
        The returned array will be "safe" in that no references to it are
        maintained by this list.  (In other words, this method must allocate
        a new array).  The caller is thus free to modify the returned array.
        
        This method acts as bridge between array-based and collection-based
        APIs.

        Returns
        - an array containing all the elements in this list
        """
        ...


    def toArray(self, a: list["T"]) -> list["T"]:
        """
        Returns an array containing all of the elements in this list in
        proper sequence (from first to last element); the runtime type of
        the returned array is that of the specified array.  If the list fits
        in the specified array, it is returned therein.  Otherwise, a new
        array is allocated with the runtime type of the specified array and
        the size of this list.
        
        If this list fits in the specified array with room to spare
        (i.e., the array has more elements than this list), the element in
        the array immediately following the end of the list is set to
        `null`.  (This is useful in determining the length of this
        list *only* if the caller knows that this list does not contain
        any null elements.)
        
        Like the .toArray() method, this method acts as bridge between
        array-based and collection-based APIs.  Further, this method allows
        precise control over the runtime type of the output array, and may,
        under certain circumstances, be used to save allocation costs.
        
        Suppose `x` is a list known to contain only strings.
        The following code can be used to dump the list into a newly
        allocated array of `String`:
        
        ``` `String[] y = x.toArray(new String[0]);````
        
        Note that `toArray(new Object[0])` is identical in function to
        `toArray()`.

        Arguments
        - a: the array into which the elements of the list are to
                 be stored, if it is big enough; otherwise, a new array of the
                 same runtime type is allocated for this purpose.

        Returns
        - an array containing all the elements in this list

        Raises
        - ArrayStoreException: if the runtime type of the specified array
                is not a supertype of the runtime type of every element in
                this list
        - NullPointerException: if the specified array is null
        """
        ...


    def get(self, index: int) -> "E":
        """
        Raises
        - IndexOutOfBoundsException: 
        """
        ...


    def set(self, index: int, element: "E") -> "E":
        """
        Replaces the element at the specified position in this list with the
        specified element.

        Raises
        - IndexOutOfBoundsException: 
        """
        ...


    def add(self, e: "E") -> bool:
        """
        Appends the specified element to the end of this list.

        Arguments
        - e: element to be appended to this list

        Returns
        - `True` (as specified by Collection.add)
        """
        ...


    def add(self, index: int, element: "E") -> None:
        """
        Inserts the specified element at the specified position in this
        list. Shifts the element currently at that position (if any) and
        any subsequent elements to the right (adds one to their indices).

        Raises
        - IndexOutOfBoundsException: 
        """
        ...


    def remove(self, index: int) -> "E":
        """
        Removes the element at the specified position in this list.
        Shifts any subsequent elements to the left (subtracts one from their
        indices).  Returns the element that was removed from the list.

        Raises
        - IndexOutOfBoundsException: 
        """
        ...


    def remove(self, o: "Object") -> bool:
        """
        Removes the first occurrence of the specified element from this list,
        if it is present.  If this list does not contain the element, it is
        unchanged.  More formally, removes the element with the lowest index
        `i` such that `Objects.equals(o, get(i))`
        (if such an element exists).  Returns `True` if this list
        contained the specified element (or equivalently, if this list
        changed as a result of the call).

        Arguments
        - o: element to be removed from this list, if present

        Returns
        - `True` if this list contained the specified element
        """
        ...


    def addIfAbsent(self, e: "E") -> bool:
        """
        Appends the element, if not present.

        Arguments
        - e: element to be added to this list, if absent

        Returns
        - `True` if the element was added
        """
        ...


    def containsAll(self, c: Iterable[Any]) -> bool:
        """
        Returns `True` if this list contains all of the elements of the
        specified collection.

        Arguments
        - c: collection to be checked for containment in this list

        Returns
        - `True` if this list contains all of the elements of the
                specified collection

        Raises
        - NullPointerException: if the specified collection is null

        See
        - .contains(Object)
        """
        ...


    def removeAll(self, c: Iterable[Any]) -> bool:
        """
        Removes from this list all of its elements that are contained in
        the specified collection. This is a particularly expensive operation
        in this class because of the need for an internal temporary array.

        Arguments
        - c: collection containing elements to be removed from this list

        Returns
        - `True` if this list changed as a result of the call

        Raises
        - ClassCastException: if the class of an element of this list
                is incompatible with the specified collection
        (<a href="/java.base/java/util/Collection.html#optional-restrictions">optional</a>)
        - NullPointerException: if this list contains a null element and the
                specified collection does not permit null elements
        (<a href="/java.base/java/util/Collection.html#optional-restrictions">optional</a>),
                or if the specified collection is null

        See
        - .remove(Object)
        """
        ...


    def retainAll(self, c: Iterable[Any]) -> bool:
        """
        Retains only the elements in this list that are contained in the
        specified collection.  In other words, removes from this list all of
        its elements that are not contained in the specified collection.

        Arguments
        - c: collection containing elements to be retained in this list

        Returns
        - `True` if this list changed as a result of the call

        Raises
        - ClassCastException: if the class of an element of this list
                is incompatible with the specified collection
        (<a href="/java.base/java/util/Collection.html#optional-restrictions">optional</a>)
        - NullPointerException: if this list contains a null element and the
                specified collection does not permit null elements
        (<a href="/java.base/java/util/Collection.html#optional-restrictions">optional</a>),
                or if the specified collection is null

        See
        - .remove(Object)
        """
        ...


    def addAllAbsent(self, c: Iterable["E"]) -> int:
        """
        Appends all of the elements in the specified collection that
        are not already contained in this list, to the end of
        this list, in the order that they are returned by the
        specified collection's iterator.

        Arguments
        - c: collection containing elements to be added to this list

        Returns
        - the number of elements added

        Raises
        - NullPointerException: if the specified collection is null

        See
        - .addIfAbsent(Object)
        """
        ...


    def clear(self) -> None:
        """
        Removes all of the elements from this list.
        The list will be empty after this call returns.
        """
        ...


    def addAll(self, c: Iterable["E"]) -> bool:
        """
        Appends all of the elements in the specified collection to the end
        of this list, in the order that they are returned by the specified
        collection's iterator.

        Arguments
        - c: collection containing elements to be added to this list

        Returns
        - `True` if this list changed as a result of the call

        Raises
        - NullPointerException: if the specified collection is null

        See
        - .add(Object)
        """
        ...


    def addAll(self, index: int, c: Iterable["E"]) -> bool:
        """
        Inserts all of the elements in the specified collection into this
        list, starting at the specified position.  Shifts the element
        currently at that position (if any) and any subsequent elements to
        the right (increases their indices).  The new elements will appear
        in this list in the order that they are returned by the
        specified collection's iterator.

        Arguments
        - index: index at which to insert the first element
               from the specified collection
        - c: collection containing elements to be added to this list

        Returns
        - `True` if this list changed as a result of the call

        Raises
        - IndexOutOfBoundsException: 
        - NullPointerException: if the specified collection is null

        See
        - .add(int,Object)
        """
        ...


    def forEach(self, action: "Consumer"["E"]) -> None:
        """
        Raises
        - NullPointerException: 
        """
        ...


    def removeIf(self, filter: "Predicate"["E"]) -> bool:
        """
        Raises
        - NullPointerException: 
        """
        ...


    def replaceAll(self, operator: "UnaryOperator"["E"]) -> None:
        ...


    def sort(self, c: "Comparator"["E"]) -> None:
        ...


    def toString(self) -> str:
        """
        Returns a string representation of this list.  The string
        representation consists of the string representations of the list's
        elements in the order they are returned by its iterator, enclosed in
        square brackets (`"[]"`).  Adjacent elements are separated by
        the characters `", "` (comma and space).  Elements are
        converted to strings as by String.valueOf(Object).

        Returns
        - a string representation of this list
        """
        ...


    def equals(self, o: "Object") -> bool:
        """
        Compares the specified object with this list for equality.
        Returns `True` if the specified object is the same object
        as this object, or if it is also a List and the sequence
        of elements returned by an List.iterator() iterator
        over the specified list is the same as the sequence returned by
        an iterator over this list.  The two sequences are considered to
        be the same if they have the same length and corresponding
        elements at the same position in the sequence are *equal*.
        Two elements `e1` and `e2` are considered
        *equal* if `Objects.equals(e1, e2)`.

        Arguments
        - o: the object to be compared for equality with this list

        Returns
        - `True` if the specified object is equal to this list
        """
        ...


    def hashCode(self) -> int:
        """
        Returns the hash code value for this list.
        
        This implementation uses the definition in List.hashCode.

        Returns
        - the hash code value for this list
        """
        ...


    def iterator(self) -> Iterator["E"]:
        """
        Returns an iterator over the elements in this list in proper sequence.
        
        The returned iterator provides a snapshot of the state of the list
        when the iterator was constructed. No synchronization is needed while
        traversing the iterator. The iterator does *NOT* support the
        `remove` method.

        Returns
        - an iterator over the elements in this list in proper sequence
        """
        ...


    def listIterator(self) -> "ListIterator"["E"]:
        """
        
        
        The returned iterator provides a snapshot of the state of the list
        when the iterator was constructed. No synchronization is needed while
        traversing the iterator. The iterator does *NOT* support the
        `remove`, `set` or `add` methods.
        """
        ...


    def listIterator(self, index: int) -> "ListIterator"["E"]:
        """
        
        
        The returned iterator provides a snapshot of the state of the list
        when the iterator was constructed. No synchronization is needed while
        traversing the iterator. The iterator does *NOT* support the
        `remove`, `set` or `add` methods.

        Raises
        - IndexOutOfBoundsException: 
        """
        ...


    def spliterator(self) -> "Spliterator"["E"]:
        """
        Returns a Spliterator over the elements in this list.
        
        The `Spliterator` reports Spliterator.IMMUTABLE,
        Spliterator.ORDERED, Spliterator.SIZED, and
        Spliterator.SUBSIZED.
        
        The spliterator provides a snapshot of the state of the list
        when the spliterator was constructed. No synchronization is needed while
        operating on the spliterator.

        Returns
        - a `Spliterator` over the elements in this list

        Since
        - 1.8
        """
        ...


    def subList(self, fromIndex: int, toIndex: int) -> list["E"]:
        """
        Returns a view of the portion of this list between
        `fromIndex`, inclusive, and `toIndex`, exclusive.
        The returned list is backed by this list, so changes in the
        returned list are reflected in this list.
        
        The semantics of the list returned by this method become
        undefined if the backing list (i.e., this list) is modified in
        any way other than via the returned list.

        Arguments
        - fromIndex: low endpoint (inclusive) of the subList
        - toIndex: high endpoint (exclusive) of the subList

        Returns
        - a view of the specified range within this list

        Raises
        - IndexOutOfBoundsException: 
        """
        ...



"""
Python module generated from Java source file org.bukkit.UnsafeValues

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.collect import Multimap
from org.bukkit import *
from org.bukkit.advancement import Advancement
from org.bukkit.attribute import Attribute
from org.bukkit.attribute import AttributeModifier
from org.bukkit.block import Biome
from org.bukkit.block.data import BlockData
from org.bukkit.damage import DamageEffect
from org.bukkit.damage import DamageSource
from org.bukkit.damage import DamageType
from org.bukkit.entity import EntityType
from org.bukkit.entity import Villager
from org.bukkit.inventory import CreativeCategory
from org.bukkit.inventory import EquipmentSlot
from org.bukkit.inventory import ItemStack
from org.bukkit.material import MaterialData
from org.bukkit.plugin import InvalidPluginException
from org.bukkit.plugin import PluginDescriptionFile
from org.bukkit.potion import PotionType
from typing import Any, Callable, Iterable, Tuple


class UnsafeValues:
    """
    This interface provides value conversions that may be specific to a
    runtime, or have arbitrary meaning (read: magic values).
    
    Their existence and behavior is not guaranteed across future versions. They
    may be poorly named, throw exceptions, have misleading parameters, or any
    other bad programming practice.
    """

    def toLegacy(self, material: "Material") -> "Material":
        ...


    def fromLegacy(self, material: "Material") -> "Material":
        ...


    def fromLegacy(self, material: "MaterialData") -> "Material":
        ...


    def fromLegacy(self, material: "MaterialData", itemPriority: bool) -> "Material":
        ...


    def fromLegacy(self, material: "Material", data: int) -> "BlockData":
        ...


    def getMaterial(self, material: str, version: int) -> "Material":
        ...


    def getDataVersion(self) -> int:
        ...


    def modifyItemStack(self, stack: "ItemStack", arguments: str) -> "ItemStack":
        ...


    def checkSupported(self, pdf: "PluginDescriptionFile") -> None:
        ...


    def processClass(self, pdf: "PluginDescriptionFile", path: str, clazz: list[int]) -> list[int]:
        ...


    def loadAdvancement(self, key: "NamespacedKey", advancement: str) -> "Advancement":
        """
        Load an advancement represented by the specified string into the server.
        The advancement format is governed by Minecraft and has no specified
        layout.
        
        It is currently a JSON object, as described by the <a href="https://minecraft.wiki/w/Advancements">Minecraft wiki</a>.
        
        Loaded advancements will be stored and persisted across server restarts
        and reloads.
        
        Callers should be prepared for Exception to be thrown.

        Arguments
        - key: the unique advancement key
        - advancement: representation of the advancement

        Returns
        - the loaded advancement or null if an error occurred
        """
        ...


    def removeAdvancement(self, key: "NamespacedKey") -> bool:
        """
        Delete an advancement which was loaded and saved by
        .loadAdvancement(org.bukkit.NamespacedKey, java.lang.String).
        
        This method will only remove advancement from persistent storage. It
        should be accompanied by a call to Server.reloadData() in order
        to fully remove it from the running instance.

        Arguments
        - key: the unique advancement key

        Returns
        - True if a file matching this key was found and deleted
        """
        ...


    def getDefaultAttributeModifiers(self, material: "Material", slot: "EquipmentSlot") -> "Multimap"["Attribute", "AttributeModifier"]:
        ...


    def getCreativeCategory(self, material: "Material") -> "CreativeCategory":
        ...


    def getBlockTranslationKey(self, material: "Material") -> str:
        ...


    def getItemTranslationKey(self, material: "Material") -> str:
        ...


    def getTranslationKey(self, entityType: "EntityType") -> str:
        ...


    def getTranslationKey(self, itemStack: "ItemStack") -> str:
        ...


    def getTranslationKey(self, attribute: "Attribute") -> str:
        ...


    def getFeatureFlag(self, key: "NamespacedKey") -> "FeatureFlag":
        ...


    def getInternalPotionData(self, key: "NamespacedKey") -> "PotionType.InternalPotionData":
        """
        Do not use, method will get removed, and the plugin won't run

        Arguments
        - key: of the potion type

        Returns
        - an internal potion data
        """
        ...


    def getDamageEffect(self, key: str) -> "DamageEffect":
        ...


    def createDamageSourceBuilder(self, damageType: "DamageType") -> "DamageSource.Builder":
        """
        Create a new DamageSource.Builder.

        Arguments
        - damageType: the DamageType to use

        Returns
        - a DamageSource.Builder
        """
        ...


    def get(self, aClass: type[Any], value: str) -> str:
        ...


    def get(self, registry: "Registry"["B"], key: "NamespacedKey") -> "B":
        ...


    def getCustomBiome(self) -> "Biome":
        ...


    def createReputationType(self, key: str) -> "Villager.ReputationType":
        ...


    def createReputationEvent(self, key: str) -> "Villager.ReputationEvent":
        ...



from .Vector3ic import Vector3ic
from .Vector2ic import Vector2ic
from .Vector3dc import Vector3dc
from .Matrix3x2d import Matrix3x2d
from .Matrix3d import Matrix3d
from .MemUtil import MemUtil
from .Vector4L import Vector4L
from .AxisAngle4f import AxisAngle4f
from .QuaterniondInterpolator import QuaterniondInterpolator
from .Matrix3f import Matrix3f
from .Matrix4x3fStack import Matrix4x3fStack
from .Matrix4x3fc import Matrix4x3fc
from .Matrix4f import Matrix4f
from .Vector3fc import Vector3fc
from .Vector4fc import Vector4fc
from .Matrix4dc import Matrix4dc
from .Runtime import Runtime
from .RoundingMode import RoundingMode
from .Vector4d import Vector4d
from .Vector3d import Vector3d
from .Matrix3fc import Matrix3fc
from .FrustumRayBuilder import FrustumRayBuilder
from .Vector2i import Vector2i
from .Matrix3x2dStack import Matrix3x2dStack
from .Matrix4d import Matrix4d
from .Vector4Lc import Vector4Lc
from .Vector3Lc import Vector3Lc
from .RayAabIntersection import RayAabIntersection
from .Interpolationf import Interpolationf
from .Vector4dc import Vector4dc
from .Vector2fc import Vector2fc
from .Interpolationd import Interpolationd
from .Vector2dc import Vector2dc
from .Matrix3fStack import Matrix3fStack
from .Vector3i import Vector3i
from .Intersectionf import Intersectionf
from .Matrix2fc import Matrix2fc
from .QuaternionfInterpolator import QuaternionfInterpolator
from .Matrix3dStack import Matrix3dStack
from .Quaterniond import Quaterniond
from .Matrix3x2f import Matrix3x2f
from .GeometryUtils import GeometryUtils
from .PolygonsIntersection import PolygonsIntersection
from .Vector4f import Vector4f
from .Vector4i import Vector4i
from .Matrix2d import Matrix2d
from .Vector2d import Vector2d
from .Matrix4dStack import Matrix4dStack
from .ConfigurationException import ConfigurationException
from .Matrix3dc import Matrix3dc
from .Matrix4fStack import Matrix4fStack
from .FrustumIntersection import FrustumIntersection
from .SimplexNoise import SimplexNoise
from .Vector2f import Vector2f
from .Matrix4x3dStack import Matrix4x3dStack
from .Matrix4x3dc import Matrix4x3dc
from .AxisAngle4d import AxisAngle4d
from .JvmciCode import JvmciCode
from .Random import Random
from .Quaternionfc import Quaternionfc
from .Quaternionf import Quaternionf
from .Matrix3x2dc import Matrix3x2dc
from .Matrix3x2fc import Matrix3x2fc
from .Matrix3x2fStack import Matrix3x2fStack
from .Vector3L import Vector3L
from .Vector2Lc import Vector2Lc
from .Vector2L import Vector2L
from .Matrix2dc import Matrix2dc
from .Vector4ic import Vector4ic
from .Matrix4x3f import Matrix4x3f
from .Matrix4x3d import Matrix4x3d
from .Matrix2f import Matrix2f
from .Options import Options
from .Vector3f import Vector3f
from .Intersectiond import Intersectiond
from .Matrix4fc import Matrix4fc
from .Math import Math
from .Quaterniondc import Quaterniondc


"""
Python module generated from Java source file org.bukkit.event.inventory.FurnaceExtractEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Bukkit
from org.bukkit import Material
from org.bukkit.block import Block
from org.bukkit.entity import Player
from org.bukkit.event.block import BlockExpEvent
from org.bukkit.event.inventory import *
from org.bukkit.material import MaterialData
from typing import Any, Callable, Iterable, Tuple


class FurnaceExtractEvent(BlockExpEvent):
    """
    This event is called when a player takes items out of the furnace
    """

    def __init__(self, player: "Player", block: "Block", itemType: "Material", itemAmount: int, exp: int):
        ...


    def getPlayer(self) -> "Player":
        """
        Get the player that triggered the event

        Returns
        - the relevant player
        """
        ...


    def getItemType(self) -> "Material":
        """
        Get the Material of the item being retrieved

        Returns
        - the material of the item
        """
        ...


    def getItemAmount(self) -> int:
        """
        Get the item count being retrieved

        Returns
        - the amount of the item
        """
        ...



"""
Python module generated from Java source file org.bukkit.persistence.PersistentDataHolder

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.persistence import *
from typing import Any, Callable, Iterable, Tuple


class PersistentDataHolder:
    """
    The PersistentDataHolder interface defines an object that can store
    custom persistent meta data on it.
    """

    def getPersistentDataContainer(self) -> "PersistentDataContainer":
        """
        Returns a custom tag container capable of storing tags on the object.
        
        Note that the tags stored on this container are all stored under their
        own custom namespace therefore modifying default tags using this
        PersistentDataHolder is impossible.

        Returns
        - the persistent metadata container
        """
        ...



"""
Python module generated from Java source file java.util.function.ObjIntConsumer

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util.function import *
from typing import Any, Callable, Iterable, Tuple


class ObjIntConsumer:
    """
    Represents an operation that accepts an object-valued and a
    `int`-valued argument, and returns no result.  This is the
    `(reference, int)` specialization of BiConsumer.
    Unlike most other functional interfaces, `ObjIntConsumer` is
    expected to operate via side-effects.
    
    This is a <a href="package-summary.html">functional interface</a>
    whose functional method is .accept(Object, int).
    
    Type `<T>`: the type of the object argument to the operation

    See
    - BiConsumer

    Since
    - 1.8
    """

    def accept(self, t: "T", value: int) -> None:
        """
        Performs this operation on the given arguments.

        Arguments
        - t: the first input argument
        - value: the second input argument
        """
        ...



from .Striped64 import Striped64
from .SipHashFunction import SipHashFunction
from .PrimitiveSink import PrimitiveSink
from .ParametricNullness import ParametricNullness
from .Murmur3_32HashFunction import Murmur3_32HashFunction
from .Murmur3_128HashFunction import Murmur3_128HashFunction
from .MessageDigestHashFunction import MessageDigestHashFunction
from .MacHashFunction import MacHashFunction
from .LongAdder import LongAdder
from .LongAddables import LongAddables
from .LongAddable import LongAddable
from .LittleEndianByteArray import LittleEndianByteArray
from .Java8Compatibility import Java8Compatibility
from .ImmutableSupplier import ImmutableSupplier
from .IgnoreJRERequirement import IgnoreJRERequirement
from .HashingOutputStream import HashingOutputStream
from .HashingInputStream import HashingInputStream
from .Hashing import Hashing
from .Hasher import Hasher
from .HashFunction import HashFunction
from .HashCode import HashCode
from .Funnels import Funnels
from .Funnel import Funnel
from .Fingerprint2011 import Fingerprint2011
from .FarmHashFingerprint64 import FarmHashFingerprint64
from .ElementTypesAreNonnullByDefault import ElementTypesAreNonnullByDefault
from .Crc32cHashFunction import Crc32cHashFunction
from .ChecksumHashFunction import ChecksumHashFunction
from .BloomFilterStrategies import BloomFilterStrategies
from .BloomFilter import BloomFilter
from .AbstractStreamingHasher import AbstractStreamingHasher
from .AbstractNonStreamingHashFunction import AbstractNonStreamingHashFunction
from .AbstractHasher import AbstractHasher
from .AbstractHashFunction import AbstractHashFunction
from .AbstractCompositeHashFunction import AbstractCompositeHashFunction
from .AbstractByteHasher import AbstractByteHasher


"""
Python module generated from Java source file com.google.common.collect.Collections2

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.base import Function
from com.google.common.base import Predicate
from com.google.common.base import Predicates
from com.google.common.collect import *
from com.google.common.math import IntMath
from com.google.common.primitives import Ints
from java.util import AbstractCollection
from java.util import Arrays
from java.util import Collections
from java.util import Comparator
from java.util import Iterator
from java.util import Spliterator
from java.util.function import Consumer
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class Collections2:
    """
    Provides static methods for working with `Collection` instances.
    
    **Java 8+ users:** several common uses for this class are now more comprehensively
    addressed by the new java.util.stream.Stream library. Read the method documentation below
    for comparisons. These methods are not being deprecated, but we gently encourage you to migrate
    to streams.

    Author(s)
    - Jared Levy

    Since
    - 2.0
    """

    @staticmethod
    def filter(unfiltered: Iterable["E"], predicate: "Predicate"["E"]) -> Iterable["E"]:
        ...


    @staticmethod
    def transform(fromCollection: Iterable["F"], function: "Function"["F", "T"]) -> Iterable["T"]:
        """
        Returns a collection that applies `function` to each element of `fromCollection`.
        The returned collection is a live view of `fromCollection`; changes to one affect the
        other.
        
        The returned collection's `add()` and `addAll()` methods throw an UnsupportedOperationException. All other collection methods are supported, as long as `fromCollection` supports them.
        
        The returned collection isn't threadsafe or serializable, even if `fromCollection` is.
        
        When a live view is *not* needed, it may be faster to copy the transformed collection
        and use the copy.
        
        If the input `Collection` is known to be a `List`, consider Lists.transform. If only an `Iterable` is available, use Iterables.transform.
        
        **`Stream` equivalent:** java.util.stream.Stream.map Stream.map.
        """
        ...


    @staticmethod
    def orderedPermutations(elements: Iterable["E"]) -> Iterable[list["E"]]:
        """
        Returns a Collection of all the permutations of the specified Iterable.
        
        *Notes:* This is an implementation of the algorithm for Lexicographical Permutations
        Generation, described in Knuth's "The Art of Computer Programming", Volume 4, Chapter 7,
        Section 7.2.1.2. The iteration order follows the lexicographical order. This means that the
        first permutation will be in ascending order, and the last will be in descending order.
        
        Duplicate elements are considered equal. For example, the list [1, 1] will have only one
        permutation, instead of two. This is why the elements have to implement Comparable.
        
        An empty iterable has only one permutation, which is an empty list.
        
        This method is equivalent to `Collections2.orderedPermutations(list,
        Ordering.natural())`.

        Arguments
        - elements: the original iterable whose elements have to be permuted.

        Returns
        - an immutable Collection containing all the different permutations of the
            original iterable.

        Raises
        - NullPointerException: if the specified iterable is null or has any null elements.

        Since
        - 12.0
        """
        ...


    @staticmethod
    def orderedPermutations(elements: Iterable["E"], comparator: "Comparator"["E"]) -> Iterable[list["E"]]:
        """
        Returns a Collection of all the permutations of the specified Iterable using
        the specified Comparator for establishing the lexicographical ordering.
        
        Examples:
        
        ````for (List<String> perm : orderedPermutations(asList("b", "c", "a"))) {
          println(perm);`
        // -> ["a", "b", "c"]
        // -> ["a", "c", "b"]
        // -> ["b", "a", "c"]
        // -> ["b", "c", "a"]
        // -> ["c", "a", "b"]
        // -> ["c", "b", "a"]
        
        for (List<Integer> perm : orderedPermutations(asList(1, 2, 2, 1))) {
          println(perm);
        }
        // -> [1, 1, 2, 2]
        // -> [1, 2, 1, 2]
        // -> [1, 2, 2, 1]
        // -> [2, 1, 1, 2]
        // -> [2, 1, 2, 1]
        // -> [2, 2, 1, 1]
        }```
        
        *Notes:* This is an implementation of the algorithm for Lexicographical Permutations
        Generation, described in Knuth's "The Art of Computer Programming", Volume 4, Chapter 7,
        Section 7.2.1.2. The iteration order follows the lexicographical order. This means that the
        first permutation will be in ascending order, and the last will be in descending order.
        
        Elements that compare equal are considered equal and no new permutations are created by
        swapping them.
        
        An empty iterable has only one permutation, which is an empty list.

        Arguments
        - elements: the original iterable whose elements have to be permuted.
        - comparator: a comparator for the iterable's elements.

        Returns
        - an immutable Collection containing all the different permutations of the
            original iterable.

        Raises
        - NullPointerException: If the specified iterable is null, has any null elements, or if
            the specified comparator is null.

        Since
        - 12.0
        """
        ...


    @staticmethod
    def permutations(elements: Iterable["E"]) -> Iterable[list["E"]]:
        """
        Returns a Collection of all the permutations of the specified Collection.
        
        *Notes:* This is an implementation of the Plain Changes algorithm for permutations
        generation, described in Knuth's "The Art of Computer Programming", Volume 4, Chapter 7,
        Section 7.2.1.2.
        
        If the input list contains equal elements, some of the generated permutations will be equal.
        
        An empty collection has only one permutation, which is an empty list.

        Arguments
        - elements: the original collection whose elements have to be permuted.

        Returns
        - an immutable Collection containing all the different permutations of the
            original collection.

        Raises
        - NullPointerException: if the specified collection is null or has any null elements.

        Since
        - 12.0
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.vehicle.VehicleMoveEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Location
from org.bukkit.entity import Vehicle
from org.bukkit.event import HandlerList
from org.bukkit.event.vehicle import *
from typing import Any, Callable, Iterable, Tuple


class VehicleMoveEvent(VehicleEvent):
    """
    Raised when a vehicle moves.
    """

    def __init__(self, vehicle: "Vehicle", from: "Location", to: "Location"):
        ...


    def getFrom(self) -> "Location":
        """
        Get the previous position.

        Returns
        - Old position.
        """
        ...


    def getTo(self) -> "Location":
        """
        Get the next position.

        Returns
        - New position.
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file org.bukkit.EntityEffect

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from enum import Enum
from org.bukkit import *
from org.bukkit.entity import Ageable
from org.bukkit.entity import ArmorStand
from org.bukkit.entity import Cat
from org.bukkit.entity import Dolphin
from org.bukkit.entity import Egg
from org.bukkit.entity import Entity
from org.bukkit.entity import EvokerFangs
from org.bukkit.entity import Firework
from org.bukkit.entity import Fox
from org.bukkit.entity import Goat
from org.bukkit.entity import Guardian
from org.bukkit.entity import Hoglin
from org.bukkit.entity import IronGolem
from org.bukkit.entity import LivingEntity
from org.bukkit.entity import Player
from org.bukkit.entity import Rabbit
from org.bukkit.entity import Ravager
from org.bukkit.entity import Sheep
from org.bukkit.entity import Sniffer
from org.bukkit.entity import Snowball
from org.bukkit.entity import Squid
from org.bukkit.entity import Tameable
from org.bukkit.entity import TippedArrow
from org.bukkit.entity import Villager
from org.bukkit.entity import Warden
from org.bukkit.entity import Witch
from org.bukkit.entity import Wolf
from org.bukkit.entity import Zoglin
from org.bukkit.entity import ZombieVillager
from org.bukkit.entity.minecart import ExplosiveMinecart
from org.bukkit.entity.minecart import SpawnerMinecart
from typing import Any, Callable, Iterable, Tuple


class EntityEffect(Enum):
    """
    A list of all Effects that can happen to entities.
    """

    ARROW_PARTICLES = (0, TippedArrow)
    """
    Colored particles from a tipped arrow.
    """
    RABBIT_JUMP = (1, Rabbit)
    """
    Rabbit jumping.
    """
    RESET_SPAWNER_MINECART_DELAY = (1, SpawnerMinecart)
    """
    Resets a spawner minecart's delay to 200. Does not effect actual spawning
    delay, only the speed at which the entity in the spawner spins
    """
    HURT = (2, LivingEntity)
    """
    When mobs get hurt.

    Deprecated
    - Use LivingEntity.playHurtAnimation(float)
    """
    DEATH = (3, Entity)
    """
    When a mob dies.
    
    **This will cause client-glitches!**

    See
    - .ENTITY_DEATH

    Deprecated
    - split into individual effects
    """
    EGG_BREAK = (3, Egg)
    """
    Spawns the egg breaking particles
    """
    SNOWBALL_BREAK = (3, Snowball)
    """
    Spawns the snowball breaking particles
    """
    ENTITY_DEATH = (3, LivingEntity)
    """
    Plays the entity death sound and animation
    
    **This will cause client-glitches!**
    """
    FANG_ATTACK = (4, EvokerFangs)
    """
    Plays the fang attack animation
    """
    HOGLIN_ATTACK = (4, Hoglin)
    """
    Plays the hoglin attack animation
    """
    IRON_GOLEN_ATTACK = (4, IronGolem)
    """
    Plays the iron golem attack animation
    """
    RAVAGER_ATTACK = (4, Ravager)
    """
    Plays the ravager attack animation
    """
    WARDEN_ATTACK = (4, Warden)
    """
    Plays the warden attack animation
    """
    ZOGLIN_ATTACK = (4, Zoglin)
    """
    Plays the zoglin attack animation
    """
    WOLF_SMOKE = (6, Tameable)
    """
    The smoke when taming an entity fails.
    """
    WOLF_HEARTS = (7, Tameable)
    """
    The hearts when taming an entity succeeds.
    """
    WOLF_SHAKE = (8, Wolf)
    """
    When a wolf shakes (after being wet).

    See
    - EntityEffect.WOLF_SHAKE_STOP
    """
    SHEEP_EAT = (10, Entity)
    """
    When an entity eats a LONG_GRASS block.

    See
    - .TNT_MINECART_IGNITE

    Deprecated
    - split into individual effects
    """
    SHEEP_EAT_GRASS = (10, Sheep)
    """
    Plays the sheep eating grass animation
    """
    TNT_MINECART_IGNITE = (10, ExplosiveMinecart)
    """
    Causes the TNT minecart to ignite, does not play the ignition sound
    
    **This will cause client-glitches!**
    """
    IRON_GOLEM_ROSE = (11, IronGolem)
    """
    When an Iron Golem gives a rose.
    """
    VILLAGER_HEART = (12, Villager)
    """
    Hearts from a villager.
    """
    VILLAGER_ANGRY = (13, Villager)
    """
    When a villager is angry.
    """
    VILLAGER_HAPPY = (14, Villager)
    """
    Happy particles from a villager.
    """
    WITCH_MAGIC = (15, Witch)
    """
    Magic particles from a witch.
    """
    ZOMBIE_TRANSFORM = (16, ZombieVillager)
    """
    When a zombie transforms into a villager by shaking violently.
    """
    FIREWORK_EXPLODE = (17, Firework)
    """
    When a firework explodes.
    """
    LOVE_HEARTS = (18, Ageable)
    """
    Hearts from a breeding entity.
    """
    SQUID_ROTATE = (19, Squid)
    """
    Resets squid rotation.
    """
    ENTITY_POOF = (20, LivingEntity)
    """
    Silverfish entering block, spawner spawning.
    """
    GUARDIAN_TARGET = (21, Guardian)
    """
    Guardian plays the attack sound effect.
    """
    SHIELD_BLOCK = (29, LivingEntity)
    """
    Shield blocks attack.
    """
    SHIELD_BREAK = (30, LivingEntity)
    """
    Shield breaks.
    """
    ARMOR_STAND_HIT = (32, ArmorStand)
    """
    Armor stand is hit.
    """
    THORNS_HURT = (33, LivingEntity)
    """
    Entity hurt by thorns attack.
    """
    IRON_GOLEM_SHEATH = (34, IronGolem)
    """
    Iron golem puts away rose.
    """
    TOTEM_RESURRECT = (35, LivingEntity)
    """
    Totem prevents entity death.
    """
    HURT_DROWN = (36, LivingEntity)
    """
    Entity hurt due to drowning damage.
    """
    HURT_EXPLOSION = (37, LivingEntity)
    """
    Entity hurt due to explosion damage.
    """
    DOLPHIN_FED = (38, Dolphin)
    """
    Dolphin has been fed and is locating a structure.
    """
    RAVAGER_STUNNED = (39, Ravager)
    """
    Ravager has been stunned for 40 ticks.
    """
    CAT_TAME_FAIL = (40, Cat)
    """
    Cat taming failed.
    """
    CAT_TAME_SUCCESS = (41, Cat)
    """
    Cat taming succeeded.
    """
    VILLAGER_SPLASH = (42, Villager)
    """
    Villager splashes particles during a raid.
    """
    PLAYER_BAD_OMEN_RAID = (43, Player)
    """
    Player's bad omen effect removed to start or increase raid difficult.
    """
    HURT_BERRY_BUSH = (44, LivingEntity)
    """
    Entity hurt due to berry bush. Prickly!
    """
    FOX_CHEW = (45, Fox)
    """
    Fox chews the food in its mouth
    """
    TELEPORT_ENDER = (46, LivingEntity)
    """
    Entity teleported as a result of chorus fruit or as an enderman
    """
    BREAK_EQUIPMENT_MAIN_HAND = (47, LivingEntity)
    """
    Entity breaks item in main hand
    """
    BREAK_EQUIPMENT_OFF_HAND = (48, LivingEntity)
    """
    Entity breaks item in off hand
    """
    BREAK_EQUIPMENT_HELMET = (49, LivingEntity)
    """
    Entity breaks item in helmet slot
    """
    BREAK_EQUIPMENT_CHESTPLATE = (50, LivingEntity)
    """
    Entity breaks item in chestplate slot
    """
    BREAK_EQUIPMENT_LEGGINGS = (51, LivingEntity)
    """
    Entity breaks item in legging slot
    """
    BREAK_EQUIPMENT_BOOTS = (52, LivingEntity)
    """
    Entity breaks item in boot slot
    """
    HONEY_BLOCK_SLIDE_PARTICLES = (53, Entity)
    """
    Spawns honey block slide particles at the entity's feet
    """
    HONEY_BLOCK_FALL_PARTICLES = (54, LivingEntity)
    """
    Spawns honey block fall particles at the entity's feet
    """
    SWAP_HAND_ITEMS = (55, LivingEntity)
    """
    Entity swaps the items in their hand and offhand
    """
    WOLF_SHAKE_STOP = (56, Wolf)
    """
    Stops a wolf that is currently shaking

    See
    - EntityEffect.WOLF_SHAKE
    """
    GOAT_LOWER_HEAD = (58, Goat)
    """
    Goat lowers its head for ramming

    See
    - .GOAT_RAISE_HEAD
    """
    GOAT_RAISE_HEAD = (59, Goat)
    """
    Goat raises its head

    See
    - .GOAT_LOWER_HEAD
    """
    SPAWN_DEATH_SMOKE = (60, LivingEntity)
    """
    Spawns death smoke particles
    """
    WARDEN_TENDRIL_SHAKE = (61, Warden)
    """
    Warden shakes its tendrils
    """
    WARDEN_SONIC_ATTACK = (62, Warden)
    """
    Warden performs sonic attack animation 
    Does not play the sound or fire the beam
    """
    SNIFFER_DIG = (63, Sniffer)
    """
    Plays sniffer digging sound 
    Sniffer must have a target and be in Sniffer.State.SEARCHING or
    Sniffer.State.DIGGING
    """


    def getData(self) -> int:
        """
        Gets the data value of this EntityEffect, may not be unique.

        Returns
        - The data value

        Deprecated
        - Magic value
        """
        ...


    def getApplicable(self) -> type["Entity"]:
        """
        Gets entity superclass which this affect is applicable to.

        Returns
        - applicable class
        """
        ...


    def isApplicableTo(self, entity: "Entity") -> bool:
        """
        Checks if this effect is applicable to the given entity.

        Arguments
        - entity: the entity to check

        Returns
        - True if applicable
        """
        ...


    def isApplicableTo(self, clazz: type["Entity"]) -> bool:
        """
        Checks if this effect is applicable to the given entity class.

        Arguments
        - clazz: the entity class to check

        Returns
        - True if applicable
        """
        ...



"""
Python module generated from Java source file org.joml.Quaternionfc

Java source file obtained from artifact joml version 1.10.8

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import *
from org.joml import *
from typing import Any, Callable, Iterable, Tuple


class Quaternionfc:
    """
    Interface to a read-only view of a quaternion of single-precision floats.

    Author(s)
    - Kai Burjack
    """

    def x(self) -> float:
        """
        Returns
        - the first component of the vector part
        """
        ...


    def y(self) -> float:
        """
        Returns
        - the second component of the vector part
        """
        ...


    def z(self) -> float:
        """
        Returns
        - the third component of the vector part
        """
        ...


    def w(self) -> float:
        """
        Returns
        - the real/scalar part of the quaternion
        """
        ...


    def normalize(self, dest: "Quaternionf") -> "Quaternionf":
        """
        Normalize this quaternion and store the result in `dest`.

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def add(self, x: float, y: float, z: float, w: float, dest: "Quaternionf") -> "Quaternionf":
        """
        Add the quaternion `(x, y, z, w)` to this quaternion and store the result in `dest`.

        Arguments
        - x: the x component of the vector part
        - y: the y component of the vector part
        - z: the z component of the vector part
        - w: the real/scalar component
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def add(self, q2: "Quaternionfc", dest: "Quaternionf") -> "Quaternionf":
        """
        Add `q2` to this quaternion and store the result in `dest`.

        Arguments
        - q2: the quaternion to add to this
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def sub(self, x: float, y: float, z: float, w: float, dest: "Quaternionf") -> "Quaternionf":
        """
        Subtract the quaternion `(x, y, z, w)` from this quaternion and store the result in `dest`.

        Arguments
        - x: the x component of the vector part
        - y: the y component of the vector part
        - z: the z component of the vector part
        - w: the real/scalar component
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def sub(self, q2: "Quaternionfc", dest: "Quaternionf") -> "Quaternionf":
        """
        Subtract `q2` from this quaternion and store the result in `dest`.

        Arguments
        - q2: the quaternion to add to this
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def angle(self) -> float:
        """
        Return the angle in radians represented by this normalized quaternion rotation.
        
        This quaternion must be .normalize(Quaternionf) normalized.

        Returns
        - the angle in radians
        """
        ...


    def get(self, dest: "Matrix3f") -> "Matrix3f":
        """
        Set the given destination matrix to the rotation represented by `this`.

        Arguments
        - dest: the matrix to write the rotation into

        Returns
        - the passed in destination

        See
        - Matrix3f.set(Quaternionfc)
        """
        ...


    def get(self, dest: "Matrix3d") -> "Matrix3d":
        """
        Set the given destination matrix to the rotation represented by `this`.

        Arguments
        - dest: the matrix to write the rotation into

        Returns
        - the passed in destination

        See
        - Matrix3d.set(Quaternionfc)
        """
        ...


    def get(self, dest: "Matrix4f") -> "Matrix4f":
        """
        Set the given destination matrix to the rotation represented by `this`.

        Arguments
        - dest: the matrix to write the rotation into

        Returns
        - the passed in destination

        See
        - Matrix4f.set(Quaternionfc)
        """
        ...


    def get(self, dest: "Matrix4d") -> "Matrix4d":
        """
        Set the given destination matrix to the rotation represented by `this`.

        Arguments
        - dest: the matrix to write the rotation into

        Returns
        - the passed in destination

        See
        - Matrix4d.set(Quaternionfc)
        """
        ...


    def get(self, dest: "Matrix4x3f") -> "Matrix4x3f":
        """
        Set the given destination matrix to the rotation represented by `this`.

        Arguments
        - dest: the matrix to write the rotation into

        Returns
        - the passed in destination

        See
        - Matrix4x3f.set(Quaternionfc)
        """
        ...


    def get(self, dest: "Matrix4x3d") -> "Matrix4x3d":
        """
        Set the given destination matrix to the rotation represented by `this`.

        Arguments
        - dest: the matrix to write the rotation into

        Returns
        - the passed in destination

        See
        - Matrix4x3d.set(Quaternionfc)
        """
        ...


    def get(self, dest: "AxisAngle4f") -> "AxisAngle4f":
        """
        Set the given AxisAngle4f to represent the rotation of
        `this` quaternion.

        Arguments
        - dest: the AxisAngle4f to set

        Returns
        - the passed in destination
        """
        ...


    def get(self, dest: "AxisAngle4d") -> "AxisAngle4d":
        """
        Set the given AxisAngle4d to represent the rotation of
        `this` quaternion.

        Arguments
        - dest: the AxisAngle4d to set

        Returns
        - the passed in destination
        """
        ...


    def get(self, dest: "Quaterniond") -> "Quaterniond":
        """
        Set the given Quaterniond to the values of `this`.

        Arguments
        - dest: the Quaterniond to set

        Returns
        - the passed in destination

        See
        - Quaterniond.set(Quaternionfc)
        """
        ...


    def get(self, dest: "Quaternionf") -> "Quaternionf":
        """
        Set the given Quaternionf to the values of `this`.

        Arguments
        - dest: the Quaternionf to set

        Returns
        - the passed in destination
        """
        ...


    def getAsMatrix3f(self, dest: "ByteBuffer") -> "ByteBuffer":
        """
        Store the 3x3 float matrix representation of `this` quaternion in column-major order into the given ByteBuffer.
        
        This is equivalent to calling: `this.get(new Matrix3f()).get(dest)`

        Arguments
        - dest: the destination buffer

        Returns
        - dest
        """
        ...


    def getAsMatrix3f(self, dest: "FloatBuffer") -> "FloatBuffer":
        """
        Store the 3x3 float matrix representation of `this` quaternion in column-major order into the given FloatBuffer.
        
        This is equivalent to calling: `this.get(new Matrix3f()).get(dest)`

        Arguments
        - dest: the destination buffer

        Returns
        - dest
        """
        ...


    def getAsMatrix4f(self, dest: "ByteBuffer") -> "ByteBuffer":
        """
        Store the 4x4 float matrix representation of `this` quaternion in column-major order into the given ByteBuffer.
        
        This is equivalent to calling: `this.get(new Matrix4f()).get(dest)`

        Arguments
        - dest: the destination buffer

        Returns
        - dest
        """
        ...


    def getAsMatrix4f(self, dest: "FloatBuffer") -> "FloatBuffer":
        """
        Store the 4x4 float matrix representation of `this` quaternion in column-major order into the given FloatBuffer.
        
        This is equivalent to calling: `this.get(new Matrix4f()).get(dest)`

        Arguments
        - dest: the destination buffer

        Returns
        - dest
        """
        ...


    def getAsMatrix4x3f(self, dest: "ByteBuffer") -> "ByteBuffer":
        """
        Store the 4x3 float matrix representation of `this` quaternion in column-major order into the given ByteBuffer.
        
        This is equivalent to calling: `this.get(new Matrix4x3f()).get(dest)`

        Arguments
        - dest: the destination buffer

        Returns
        - dest
        """
        ...


    def getAsMatrix4x3f(self, dest: "FloatBuffer") -> "FloatBuffer":
        """
        Store the 4x3 float matrix representation of `this` quaternion in column-major order into the given FloatBuffer.
        
        This is equivalent to calling: `this.get(new Matrix4x3f()).get(dest)`

        Arguments
        - dest: the destination buffer

        Returns
        - dest
        """
        ...


    def mul(self, q: "Quaternionfc", dest: "Quaternionf") -> "Quaternionf":
        """
        Multiply this quaternion by `q` and store the result in `dest`.
        
        If `T` is `this` and `Q` is the given
        quaternion, then the resulting quaternion `R` is:
        
        `R = T * Q`
        
        So, this method uses post-multiplication like the matrix classes, resulting in a
        vector to be transformed by `Q` first, and then by `T`.

        Arguments
        - q: the quaternion to multiply `this` by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mul(self, qx: float, qy: float, qz: float, qw: float, dest: "Quaternionf") -> "Quaternionf":
        """
        Multiply this quaternion by the quaternion represented via `(qx, qy, qz, qw)` and store the result in `dest`.
        
        If `T` is `this` and `Q` is the given
        quaternion, then the resulting quaternion `R` is:
        
        `R = T * Q`
        
        So, this method uses post-multiplication like the matrix classes, resulting in a
        vector to be transformed by `Q` first, and then by `T`.

        Arguments
        - qx: the x component of the quaternion to multiply `this` by
        - qy: the y component of the quaternion to multiply `this` by
        - qz: the z component of the quaternion to multiply `this` by
        - qw: the w component of the quaternion to multiply `this` by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mul(self, f: float, dest: "Quaternionf") -> "Quaternionf":
        """
        Multiply this quaternion by the given scalar and store the result in `dest`.
        
        This method multiplies all the four components by the specified scalar.

        Arguments
        - f: the factor to multiply all components by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def premul(self, q: "Quaternionfc", dest: "Quaternionf") -> "Quaternionf":
        """
        Pre-multiply this quaternion by `q` and store the result in `dest`.
        
        If `T` is `this` and `Q` is the given quaternion, then the resulting quaternion `R` is:
        
        `R = Q * T`
        
        So, this method uses pre-multiplication, resulting in a vector to be transformed by `T` first, and then by `Q`.

        Arguments
        - q: the quaternion to pre-multiply `this` by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def premul(self, qx: float, qy: float, qz: float, qw: float, dest: "Quaternionf") -> "Quaternionf":
        """
        Pre-multiply this quaternion by the quaternion represented via `(qx, qy, qz, qw)` and store the result in `dest`.
        
        If `T` is `this` and `Q` is the given quaternion, then the resulting quaternion `R` is:
        
        `R = Q * T`
        
        So, this method uses pre-multiplication, resulting in a vector to be transformed by `T` first, and then by `Q`.

        Arguments
        - qx: the x component of the quaternion to multiply `this` by
        - qy: the y component of the quaternion to multiply `this` by
        - qz: the z component of the quaternion to multiply `this` by
        - qw: the w component of the quaternion to multiply `this` by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transform(self, vec: "Vector3f") -> "Vector3f":
        """
        Transform the given vector by this quaternion.
        
        This will apply the rotation described by this quaternion to the given vector.

        Arguments
        - vec: the vector to transform

        Returns
        - vec
        """
        ...


    def transformInverse(self, vec: "Vector3f") -> "Vector3f":
        """
        Transform the given vector by the inverse of this quaternion.
        
        This will apply the rotation described by this quaternion to the given vector.

        Arguments
        - vec: the vector to transform

        Returns
        - vec
        """
        ...


    def transformUnit(self, vec: "Vector3f") -> "Vector3f":
        """
        Transform the given vector by this unit quaternion.
        
        This will apply the rotation described by this quaternion to the given vector.
        
        This method is only applicable when `this` is a unit quaternion.

        Arguments
        - vec: the vector to transform

        Returns
        - vec
        """
        ...


    def transformPositiveX(self, dest: "Vector3f") -> "Vector3f":
        """
        Transform the vector `(1, 0, 0)` by this quaternion.

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformPositiveX(self, dest: "Vector4f") -> "Vector4f":
        """
        Transform the vector `(1, 0, 0)` by this quaternion.
        
        Only the first three components of the given 4D vector are modified.

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformUnitPositiveX(self, dest: "Vector3f") -> "Vector3f":
        """
        Transform the vector `(1, 0, 0)` by this unit quaternion.
        
        This method is only applicable when `this` is a unit quaternion.
        
        Reference: <a href="https://de.mathworks.com/help/aerotbx/ug/quatrotate.html?requestedDomain=True">https://de.mathworks.com/</a>

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformUnitPositiveX(self, dest: "Vector4f") -> "Vector4f":
        """
        Transform the vector `(1, 0, 0)` by this unit quaternion.
        
        Only the first three components of the given 4D vector are modified.
        
        This method is only applicable when `this` is a unit quaternion.
        
        Reference: <a href="https://de.mathworks.com/help/aerotbx/ug/quatrotate.html?requestedDomain=True">https://de.mathworks.com/</a>

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformPositiveY(self, dest: "Vector3f") -> "Vector3f":
        """
        Transform the vector `(0, 1, 0)` by this quaternion.

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformPositiveY(self, dest: "Vector4f") -> "Vector4f":
        """
        Transform the vector `(0, 1, 0)` by this quaternion.
        
        Only the first three components of the given 4D vector are modified.

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformUnitPositiveY(self, dest: "Vector3f") -> "Vector3f":
        """
        Transform the vector `(0, 1, 0)` by this unit quaternion.
        
        This method is only applicable when `this` is a unit quaternion.
        
        Reference: <a href="https://de.mathworks.com/help/aerotbx/ug/quatrotate.html?requestedDomain=True">https://de.mathworks.com/</a>

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformUnitPositiveY(self, dest: "Vector4f") -> "Vector4f":
        """
        Transform the vector `(0, 1, 0)` by this unit quaternion.
        
        Only the first three components of the given 4D vector are modified.
        
        This method is only applicable when `this` is a unit quaternion.
        
        Reference: <a href="https://de.mathworks.com/help/aerotbx/ug/quatrotate.html?requestedDomain=True">https://de.mathworks.com/</a>

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformPositiveZ(self, dest: "Vector3f") -> "Vector3f":
        """
        Transform the vector `(0, 0, 1)` by this quaternion.

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformPositiveZ(self, dest: "Vector4f") -> "Vector4f":
        """
        Transform the vector `(0, 0, 1)` by this quaternion.
        
        Only the first three components of the given 4D vector are modified.

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformUnitPositiveZ(self, dest: "Vector3f") -> "Vector3f":
        """
        Transform the vector `(0, 0, 1)` by this unit quaternion.
        
        This method is only applicable when `this` is a unit quaternion.
        
        Reference: <a href="https://de.mathworks.com/help/aerotbx/ug/quatrotate.html?requestedDomain=True">https://de.mathworks.com/</a>

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformUnitPositiveZ(self, dest: "Vector4f") -> "Vector4f":
        """
        Transform the vector `(0, 0, 1)` by this unit quaternion.
        
        Only the first three components of the given 4D vector are modified.
        
        This method is only applicable when `this` is a unit quaternion.
        
        Reference: <a href="https://de.mathworks.com/help/aerotbx/ug/quatrotate.html?requestedDomain=True">https://de.mathworks.com/</a>

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transform(self, vec: "Vector4f") -> "Vector4f":
        """
        Transform the given vector by this quaternion.
        
        This will apply the rotation described by this quaternion to the given vector.
        
        Only the first three components of the given 4D vector are being used and modified.

        Arguments
        - vec: the vector to transform

        Returns
        - vec
        """
        ...


    def transformInverse(self, vec: "Vector4f") -> "Vector4f":
        """
        Transform the given vector by the inverse of this quaternion.
        
        This will apply the rotation described by this quaternion to the given vector.
        
        Only the first three components of the given 4D vector are being used and modified.

        Arguments
        - vec: the vector to transform

        Returns
        - vec
        """
        ...


    def transform(self, vec: "Vector3fc", dest: "Vector3f") -> "Vector3f":
        """
        Transform the given vector by this quaternion
        and store the result in `dest`.
        
        This will apply the rotation described by this quaternion to the given vector.

        Arguments
        - vec: the vector to transform
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformInverse(self, vec: "Vector3fc", dest: "Vector3f") -> "Vector3f":
        """
        Transform the given vector by the inverse of quaternion
        and store the result in `dest`.
        
        This will apply the rotation described by this quaternion to the given vector.

        Arguments
        - vec: the vector to transform
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transform(self, x: float, y: float, z: float, dest: "Vector3f") -> "Vector3f":
        """
        Transform the given vector `(x, y, z)` by this quaternion
        and store the result in `dest`.
        
        This will apply the rotation described by this quaternion to the given vector.

        Arguments
        - x: the x coordinate of the vector to transform
        - y: the y coordinate of the vector to transform
        - z: the z coordinate of the vector to transform
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transform(self, x: float, y: float, z: float, dest: "Vector3d") -> "Vector3d":
        """
        Transform the given vector `(x, y, z)` by this quaternion
        and store the result in `dest`.
        
        This will apply the rotation described by this quaternion to the given vector.

        Arguments
        - x: the x coordinate of the vector to transform
        - y: the y coordinate of the vector to transform
        - z: the z coordinate of the vector to transform
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformInverse(self, x: float, y: float, z: float, dest: "Vector3f") -> "Vector3f":
        """
        Transform the given vector `(x, y, z)` by the inverse of this quaternion
        and store the result in `dest`.
        
        This will apply the rotation described by this quaternion to the given vector.

        Arguments
        - x: the x coordinate of the vector to transform
        - y: the y coordinate of the vector to transform
        - z: the z coordinate of the vector to transform
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformInverse(self, x: float, y: float, z: float, dest: "Vector3d") -> "Vector3d":
        """
        Transform the given vector `(x, y, z)` by the inverse of this quaternion
        and store the result in `dest`.
        
        This will apply the rotation described by this quaternion to the given vector.

        Arguments
        - x: the x coordinate of the vector to transform
        - y: the y coordinate of the vector to transform
        - z: the z coordinate of the vector to transform
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformInverseUnit(self, vec: "Vector3f") -> "Vector3f":
        """
        Transform the given vector by the inverse of this unit quaternion.
        
        This will apply the rotation described by this quaternion to the given vector.
        
        This method is only applicable when `this` is a unit quaternion.

        Arguments
        - vec: the vector to transform

        Returns
        - vec
        """
        ...


    def transformUnit(self, vec: "Vector3fc", dest: "Vector3f") -> "Vector3f":
        """
        Transform the given vector by this unit quaternion
        and store the result in `dest`.
        
        This will apply the rotation described by this quaternion to the given vector.
        
        This method is only applicable when `this` is a unit quaternion.

        Arguments
        - vec: the vector to transform
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformInverseUnit(self, vec: "Vector3fc", dest: "Vector3f") -> "Vector3f":
        """
        Transform the given vector by the inverse of this unit quaternion
        and store the result in `dest`.
        
        This will apply the rotation described by this quaternion to the given vector.
        
        This method is only applicable when `this` is a unit quaternion.

        Arguments
        - vec: the vector to transform
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformUnit(self, x: float, y: float, z: float, dest: "Vector3f") -> "Vector3f":
        """
        Transform the given vector `(x, y, z)` by this unit quaternion
        and store the result in `dest`.
        
        This will apply the rotation described by this quaternion to the given vector.
        
        This method is only applicable when `this` is a unit quaternion.

        Arguments
        - x: the x coordinate of the vector to transform
        - y: the y coordinate of the vector to transform
        - z: the z coordinate of the vector to transform
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformUnit(self, x: float, y: float, z: float, dest: "Vector3d") -> "Vector3d":
        """
        Transform the given vector `(x, y, z)` by this unit quaternion
        and store the result in `dest`.
        
        This will apply the rotation described by this quaternion to the given vector.
        
        This method is only applicable when `this` is a unit quaternion.

        Arguments
        - x: the x coordinate of the vector to transform
        - y: the y coordinate of the vector to transform
        - z: the z coordinate of the vector to transform
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformInverseUnit(self, x: float, y: float, z: float, dest: "Vector3f") -> "Vector3f":
        """
        Transform the given vector `(x, y, z)` by the inverse of this unit quaternion
        and store the result in `dest`.
        
        This will apply the rotation described by this quaternion to the given vector.
        
        This method is only applicable when `this` is a unit quaternion.

        Arguments
        - x: the x coordinate of the vector to transform
        - y: the y coordinate of the vector to transform
        - z: the z coordinate of the vector to transform
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformInverseUnit(self, x: float, y: float, z: float, dest: "Vector3d") -> "Vector3d":
        """
        Transform the given vector `(x, y, z)` by the inverse of this unit quaternion
        and store the result in `dest`.
        
        This will apply the rotation described by this quaternion to the given vector.
        
        This method is only applicable when `this` is a unit quaternion.

        Arguments
        - x: the x coordinate of the vector to transform
        - y: the y coordinate of the vector to transform
        - z: the z coordinate of the vector to transform
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transform(self, vec: "Vector4fc", dest: "Vector4f") -> "Vector4f":
        """
        Transform the given vector by this quaternion and store the result in `dest`.
        
        This will apply the rotation described by this quaternion to the given vector.
        
        Only the first three components of the given 4D vector are being used and set on the destination.

        Arguments
        - vec: the vector to transform
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformInverse(self, vec: "Vector4fc", dest: "Vector4f") -> "Vector4f":
        """
        Transform the given vector by the inverse of this quaternion and store the result in `dest`.
        
        This will apply the rotation described by this quaternion to the given vector.
        
        Only the first three components of the given 4D vector are being used and set on the destination.

        Arguments
        - vec: the vector to transform
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transform(self, x: float, y: float, z: float, dest: "Vector4f") -> "Vector4f":
        """
        Transform the given vector `(x, y, z)` by this quaternion and store the result in `dest`.
        
        This will apply the rotation described by this quaternion to the given vector.

        Arguments
        - x: the x coordinate of the vector to transform
        - y: the y coordinate of the vector to transform
        - z: the z coordinate of the vector to transform
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformInverse(self, x: float, y: float, z: float, dest: "Vector4f") -> "Vector4f":
        """
        Transform the given vector `(x, y, z)` by the inverse of
        this quaternion and store the result in `dest`.
        
        This will apply the rotation described by this quaternion to the given vector.

        Arguments
        - x: the x coordinate of the vector to transform
        - y: the y coordinate of the vector to transform
        - z: the z coordinate of the vector to transform
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformUnit(self, vec: "Vector4fc", dest: "Vector4f") -> "Vector4f":
        """
        Transform the given vector by this unit quaternion and store the result in `dest`.
        
        This will apply the rotation described by this quaternion to the given vector.
        
        Only the first three components of the given 4D vector are being used and set on the destination.
        
        This method is only applicable when `this` is a unit quaternion.

        Arguments
        - vec: the vector to transform
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformUnit(self, vec: "Vector4f") -> "Vector4f":
        """
        Transform the given vector by this unit quaternion.
        
        This will apply the rotation described by this quaternion to the given vector.
        
        Only the first three components of the given 4D vector are being used and set on the destination.
        
        This method is only applicable when `this` is a unit quaternion.

        Arguments
        - vec: the vector to transform

        Returns
        - vec
        """
        ...


    def transformInverseUnit(self, vec: "Vector4f") -> "Vector4f":
        """
        Transform the given vector by the inverse of this unit quaternion.
        
        This will apply the rotation described by this quaternion to the given vector.
        
        Only the first three components of the given 4D vector are being used and set on the destination.
        
        This method is only applicable when `this` is a unit quaternion.

        Arguments
        - vec: the vector to transform

        Returns
        - vec
        """
        ...


    def transformInverseUnit(self, vec: "Vector4fc", dest: "Vector4f") -> "Vector4f":
        """
        Transform the given vector by the inverse of this unit quaternion and store the result in `dest`.
        
        This will apply the rotation described by this quaternion to the given vector.
        
        Only the first three components of the given 4D vector are being used and set on the destination.
        
        This method is only applicable when `this` is a unit quaternion.

        Arguments
        - vec: the vector to transform
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformUnit(self, x: float, y: float, z: float, dest: "Vector4f") -> "Vector4f":
        """
        Transform the given vector `(x, y, z)` by this unit quaternion and store the result in `dest`.
        
        This will apply the rotation described by this quaternion to the given vector.
        
        This method is only applicable when `this` is a unit quaternion.

        Arguments
        - x: the x coordinate of the vector to transform
        - y: the y coordinate of the vector to transform
        - z: the z coordinate of the vector to transform
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformInverseUnit(self, x: float, y: float, z: float, dest: "Vector4f") -> "Vector4f":
        """
        Transform the given vector `(x, y, z)` by the inverse of
        this unit quaternion and store the result in `dest`.
        
        This will apply the rotation described by this quaternion to the given vector.
        
        This method is only applicable when `this` is a unit quaternion.

        Arguments
        - x: the x coordinate of the vector to transform
        - y: the y coordinate of the vector to transform
        - z: the z coordinate of the vector to transform
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transform(self, vec: "Vector3d") -> "Vector3d":
        """
        Transform the given vector by this quaternion.
        
        This will apply the rotation described by this quaternion to the given vector.

        Arguments
        - vec: the vector to transform

        Returns
        - vec
        """
        ...


    def transformInverse(self, vec: "Vector3d") -> "Vector3d":
        """
        Transform the given vector by the inverse of this quaternion.
        
        This will apply the rotation described by this quaternion to the given vector.

        Arguments
        - vec: the vector to transform

        Returns
        - vec
        """
        ...


    def transformPositiveX(self, dest: "Vector3d") -> "Vector3d":
        """
        Transform the vector `(1, 0, 0)` by this quaternion.

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformPositiveX(self, dest: "Vector4d") -> "Vector4d":
        """
        Transform the vector `(1, 0, 0)` by this quaternion.
        
        Only the first three components of the given 4D vector are modified.

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformUnitPositiveX(self, dest: "Vector3d") -> "Vector3d":
        """
        Transform the vector `(1, 0, 0)` by this unit quaternion.
        
        This method is only applicable when `this` is a unit quaternion.
        
        Reference: <a href="https://de.mathworks.com/help/aerotbx/ug/quatrotate.html?requestedDomain=True">https://de.mathworks.com/</a>

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformUnitPositiveX(self, dest: "Vector4d") -> "Vector4d":
        """
        Transform the vector `(1, 0, 0)` by this unit quaternion.
        
        Only the first three components of the given 4D vector are modified.
        
        This method is only applicable when `this` is a unit quaternion.
        
        Reference: <a href="https://de.mathworks.com/help/aerotbx/ug/quatrotate.html?requestedDomain=True">https://de.mathworks.com/</a>

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformPositiveY(self, dest: "Vector3d") -> "Vector3d":
        """
        Transform the vector `(0, 1, 0)` by this quaternion.

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformPositiveY(self, dest: "Vector4d") -> "Vector4d":
        """
        Transform the vector `(0, 1, 0)` by this quaternion.
        
        Only the first three components of the given 4D vector are modified.

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformUnitPositiveY(self, dest: "Vector3d") -> "Vector3d":
        """
        Transform the vector `(0, 1, 0)` by this unit quaternion.
        
        This method is only applicable when `this` is a unit quaternion.
        
        Reference: <a href="https://de.mathworks.com/help/aerotbx/ug/quatrotate.html?requestedDomain=True">https://de.mathworks.com/</a>

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformUnitPositiveY(self, dest: "Vector4d") -> "Vector4d":
        """
        Transform the vector `(0, 1, 0)` by this unit quaternion.
        
        Only the first three components of the given 4D vector are modified.
        
        This method is only applicable when `this` is a unit quaternion.
        
        Reference: <a href="https://de.mathworks.com/help/aerotbx/ug/quatrotate.html?requestedDomain=True">https://de.mathworks.com/</a>

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformPositiveZ(self, dest: "Vector3d") -> "Vector3d":
        """
        Transform the vector `(0, 0, 1)` by this quaternion.

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformPositiveZ(self, dest: "Vector4d") -> "Vector4d":
        """
        Transform the vector `(0, 0, 1)` by this quaternion.
        
        Only the first three components of the given 4D vector are modified.

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformUnitPositiveZ(self, dest: "Vector3d") -> "Vector3d":
        """
        Transform the vector `(0, 0, 1)` by this unit quaternion.
        
        This method is only applicable when `this` is a unit quaternion.
        
        Reference: <a href="https://de.mathworks.com/help/aerotbx/ug/quatrotate.html?requestedDomain=True">https://de.mathworks.com/</a>

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformUnitPositiveZ(self, dest: "Vector4d") -> "Vector4d":
        """
        Transform the vector `(0, 0, 1)` by this unit quaternion.
        
        Only the first three components of the given 4D vector are modified.
        
        This method is only applicable when `this` is a unit quaternion.
        
        Reference: <a href="https://de.mathworks.com/help/aerotbx/ug/quatrotate.html?requestedDomain=True">https://de.mathworks.com/</a>

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transform(self, vec: "Vector4d") -> "Vector4d":
        """
        Transform the given vector by this quaternion.
        
        This will apply the rotation described by this quaternion to the given vector.
        
        Only the first three components of the given 4D vector are being used and modified.

        Arguments
        - vec: the vector to transform

        Returns
        - vec
        """
        ...


    def transformInverse(self, vec: "Vector4d") -> "Vector4d":
        """
        Transform the given vector by the inverse of this quaternion.
        
        This will apply the rotation described by this quaternion to the given vector.
        
        Only the first three components of the given 4D vector are being used and modified.

        Arguments
        - vec: the vector to transform

        Returns
        - vec
        """
        ...


    def transform(self, vec: "Vector3dc", dest: "Vector3d") -> "Vector3d":
        """
        Transform the given vector by this quaternion and store the result in `dest`.
        
        This will apply the rotation described by this quaternion to the given vector.

        Arguments
        - vec: the vector to transform
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformInverse(self, vec: "Vector3dc", dest: "Vector3d") -> "Vector3d":
        """
        Transform the given vector by the inverse of this quaternion and store the result in `dest`.
        
        This will apply the rotation described by this quaternion to the given vector.

        Arguments
        - vec: the vector to transform
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transform(self, x: float, y: float, z: float, dest: "Vector3d") -> "Vector3d":
        """
        Transform the given vector `(x, y, z)` by this quaternion and store the result in `dest`.
        
        This will apply the rotation described by this quaternion to the given vector.

        Arguments
        - x: the x coordinate of the vector to transform
        - y: the y coordinate of the vector to transform
        - z: the z coordinate of the vector to transform
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformInverse(self, x: float, y: float, z: float, dest: "Vector3d") -> "Vector3d":
        """
        Transform the given vector `(x, y, z)` by the inverse of
        this quaternion and store the result in `dest`.
        
        This will apply the rotation described by this quaternion to the given vector.

        Arguments
        - x: the x coordinate of the vector to transform
        - y: the y coordinate of the vector to transform
        - z: the z coordinate of the vector to transform
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transform(self, vec: "Vector4dc", dest: "Vector4d") -> "Vector4d":
        """
        Transform the given vector by this quaternion and store the result in `dest`.
        
        This will apply the rotation described by this quaternion to the given vector.
        
        Only the first three components of the given 4D vector are being used and set on the destination.

        Arguments
        - vec: the vector to transform
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformInverse(self, vec: "Vector4dc", dest: "Vector4d") -> "Vector4d":
        """
        Transform the given vector by the inverse of this quaternion and store the result in `dest`.
        
        This will apply the rotation described by this quaternion to the given vector.
        
        Only the first three components of the given 4D vector are being used and set on the destination.

        Arguments
        - vec: the vector to transform
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transform(self, x: float, y: float, z: float, dest: "Vector4d") -> "Vector4d":
        """
        Transform the given vector `(x, y, z)` by this quaternion and store the result in `dest`.
        
        This will apply the rotation described by this quaternion to the given vector.

        Arguments
        - x: the x coordinate of the vector to transform
        - y: the y coordinate of the vector to transform
        - z: the z coordinate of the vector to transform
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformInverse(self, x: float, y: float, z: float, dest: "Vector4d") -> "Vector4d":
        """
        Transform the given vector `(x, y, z)` by the inverse of
        this quaternion and store the result in `dest`.
        
        This will apply the rotation described by this quaternion to the given vector.

        Arguments
        - x: the x coordinate of the vector to transform
        - y: the y coordinate of the vector to transform
        - z: the z coordinate of the vector to transform
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformUnit(self, vec: "Vector4d") -> "Vector4d":
        """
        Transform the given vector by this unit quaternion.
        
        This will apply the rotation described by this quaternion to the given vector.
        
        Only the first three components of the given 4D vector are being used and modified.
        
        This method is only applicable when `this` is a unit quaternion.

        Arguments
        - vec: the vector to transform

        Returns
        - vec
        """
        ...


    def transformInverseUnit(self, vec: "Vector4d") -> "Vector4d":
        """
        Transform the given vector by the inverse of this unit quaternion.
        
        This will apply the rotation described by this quaternion to the given vector.
        
        Only the first three components of the given 4D vector are being used and modified.
        
        This method is only applicable when `this` is a unit quaternion.

        Arguments
        - vec: the vector to transform

        Returns
        - vec
        """
        ...


    def transformUnit(self, vec: "Vector3dc", dest: "Vector3d") -> "Vector3d":
        """
        Transform the given vector by this unit quaternion and store the result in `dest`.
        
        This will apply the rotation described by this quaternion to the given vector.
        
        This method is only applicable when `this` is a unit quaternion.

        Arguments
        - vec: the vector to transform
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformInverseUnit(self, vec: "Vector3dc", dest: "Vector3d") -> "Vector3d":
        """
        Transform the given vector by the inverse of this unit quaternion and store the result in `dest`.
        
        This will apply the rotation described by this quaternion to the given vector.
        
        This method is only applicable when `this` is a unit quaternion.

        Arguments
        - vec: the vector to transform
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformUnit(self, x: float, y: float, z: float, dest: "Vector3d") -> "Vector3d":
        """
        Transform the given vector `(x, y, z)` by this unit quaternion and store the result in `dest`.
        
        This will apply the rotation described by this quaternion to the given vector.
        
        This method is only applicable when `this` is a unit quaternion.

        Arguments
        - x: the x coordinate of the vector to transform
        - y: the y coordinate of the vector to transform
        - z: the z coordinate of the vector to transform
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformInverseUnit(self, x: float, y: float, z: float, dest: "Vector3d") -> "Vector3d":
        """
        Transform the given vector `(x, y, z)` by the inverse of
        this unit quaternion and store the result in `dest`.
        
        This will apply the rotation described by this quaternion to the given vector.
        
        This method is only applicable when `this` is a unit quaternion.

        Arguments
        - x: the x coordinate of the vector to transform
        - y: the y coordinate of the vector to transform
        - z: the z coordinate of the vector to transform
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformUnit(self, vec: "Vector4dc", dest: "Vector4d") -> "Vector4d":
        """
        Transform the given vector by this unit quaternion and store the result in `dest`.
        
        This will apply the rotation described by this quaternion to the given vector.
        
        Only the first three components of the given 4D vector are being used and set on the destination.
        
        This method is only applicable when `this` is a unit quaternion.

        Arguments
        - vec: the vector to transform
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformInverseUnit(self, vec: "Vector4dc", dest: "Vector4d") -> "Vector4d":
        """
        Transform the given vector by the inverse of this unit quaternion and store the result in `dest`.
        
        This will apply the rotation described by this quaternion to the given vector.
        
        Only the first three components of the given 4D vector are being used and set on the destination.
        
        This method is only applicable when `this` is a unit quaternion.

        Arguments
        - vec: the vector to transform
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformUnit(self, x: float, y: float, z: float, dest: "Vector4d") -> "Vector4d":
        """
        Transform the given vector `(x, y, z)` by this unit quaternion and store the result in `dest`.
        
        This will apply the rotation described by this quaternion to the given vector.
        
        This method is only applicable when `this` is a unit quaternion.

        Arguments
        - x: the x coordinate of the vector to transform
        - y: the y coordinate of the vector to transform
        - z: the z coordinate of the vector to transform
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformInverseUnit(self, x: float, y: float, z: float, dest: "Vector4d") -> "Vector4d":
        """
        Transform the given vector `(x, y, z)` by the inverse of
        this unit quaternion and store the result in `dest`.
        
        This will apply the rotation described by this quaternion to the given vector.
        
        This method is only applicable when `this` is a unit quaternion.

        Arguments
        - x: the x coordinate of the vector to transform
        - y: the y coordinate of the vector to transform
        - z: the z coordinate of the vector to transform
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def invert(self, dest: "Quaternionf") -> "Quaternionf":
        """
        Invert this quaternion and store the .normalize(Quaternionf) normalized result in `dest`.
        
        If this quaternion is already normalized, then .conjugate(Quaternionf) should be used instead.

        Arguments
        - dest: will hold the result

        Returns
        - dest

        See
        - .conjugate(Quaternionf)
        """
        ...


    def div(self, b: "Quaternionfc", dest: "Quaternionf") -> "Quaternionf":
        """
        Divide `this` quaternion by `b` and store the result in `dest`.
        
        The division expressed using the inverse is performed in the following way:
        
        `dest = this * b^-1`, where `b^-1` is the inverse of `b`.

        Arguments
        - b: the Quaternionfc to divide this by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def div(self, f: float, dest: "Quaternionf") -> "Quaternionf":
        """
        Divide this quaternion by the given scalar and store the result in `dest`.
        
        This method divides all the four components by the specified scalar.

        Arguments
        - f: the factor to divide all components by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def conjugate(self, dest: "Quaternionf") -> "Quaternionf":
        """
        Conjugate this quaternion and store the result in `dest`.

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def rotateXYZ(self, angleX: float, angleY: float, angleZ: float, dest: "Quaternionf") -> "Quaternionf":
        """
        Apply a rotation to `this` quaternion rotating the given radians about the cartesian base unit axes,
        called the euler angles using rotation sequence `XYZ` and store the result in `dest`.
        
        This method is equivalent to calling: `rotateX(angleX, dest).rotateY(angleY).rotateZ(angleZ)`
        
        If `Q` is `this` quaternion and `R` the quaternion representing the 
        specified rotation, then the new quaternion will be `Q * R`. So when transforming a
        vector `v` with the new quaternion by using `Q * R * v`, the
        rotation added by this method will be applied first!

        Arguments
        - angleX: the angle in radians to rotate about the x axis
        - angleY: the angle in radians to rotate about the y axis
        - angleZ: the angle in radians to rotate about the z axis
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def rotateZYX(self, angleZ: float, angleY: float, angleX: float, dest: "Quaternionf") -> "Quaternionf":
        """
        Apply a rotation to `this` quaternion rotating the given radians about the cartesian base unit axes,
        called the euler angles, using the rotation sequence `ZYX` and store the result in `dest`.
        
        This method is equivalent to calling: `rotateZ(angleZ, dest).rotateY(angleY).rotateX(angleX)`
        
        If `Q` is `this` quaternion and `R` the quaternion representing the 
        specified rotation, then the new quaternion will be `Q * R`. So when transforming a
        vector `v` with the new quaternion by using `Q * R * v`, the
        rotation added by this method will be applied first!

        Arguments
        - angleZ: the angle in radians to rotate about the z axis
        - angleY: the angle in radians to rotate about the y axis
        - angleX: the angle in radians to rotate about the x axis
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def rotateYXZ(self, angleY: float, angleX: float, angleZ: float, dest: "Quaternionf") -> "Quaternionf":
        """
        Apply a rotation to `this` quaternion rotating the given radians about the cartesian base unit axes,
        called the euler angles, using the rotation sequence `YXZ` and store the result in `dest`.
        
        This method is equivalent to calling: `rotateY(angleY, dest).rotateX(angleX).rotateZ(angleZ)`
        
        If `Q` is `this` quaternion and `R` the quaternion representing the 
        specified rotation, then the new quaternion will be `Q * R`. So when transforming a
        vector `v` with the new quaternion by using `Q * R * v`, the
        rotation added by this method will be applied first!

        Arguments
        - angleY: the angle in radians to rotate about the y axis
        - angleX: the angle in radians to rotate about the x axis
        - angleZ: the angle in radians to rotate about the z axis
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def getEulerAnglesXYZ(self, eulerAngles: "Vector3f") -> "Vector3f":
        """
        Get the euler angles in radians in rotation sequence `XYZ` of this quaternion and store them in the 
        provided parameter `eulerAngles`.
        
        The Euler angles are always returned as the angle around X in the Vector3f.x field, the angle around Y in the Vector3f.y
        field and the angle around Z in the Vector3f.z field of the supplied vector.

        Arguments
        - eulerAngles: will hold the euler angles in radians

        Returns
        - the passed in vector
        """
        ...


    def getEulerAnglesZYX(self, eulerAngles: "Vector3f") -> "Vector3f":
        """
        Get the euler angles in radians in rotation sequence `ZYX` of this quaternion and store them in the 
        provided parameter `eulerAngles`.
        
        The Euler angles are always returned as the angle around X in the Vector3f.x field, the angle around Y in the Vector3f.y
        field and the angle around Z in the Vector3f.z field of the supplied vector.

        Arguments
        - eulerAngles: will hold the euler angles in radians

        Returns
        - the passed in vector
        """
        ...


    def getEulerAnglesZXY(self, eulerAngles: "Vector3f") -> "Vector3f":
        """
        Get the euler angles in radians in rotation sequence `ZXY` of this quaternion and store them in the 
        provided parameter `eulerAngles`.
        
        The Euler angles are always returned as the angle around X in the Vector3f.x field, the angle around Y in the Vector3f.y
        field and the angle around Z in the Vector3f.z field of the supplied vector.

        Arguments
        - eulerAngles: will hold the euler angles in radians

        Returns
        - the passed in vector
        """
        ...


    def getEulerAnglesYXZ(self, eulerAngles: "Vector3f") -> "Vector3f":
        """
        Get the euler angles in radians in rotation sequence `YXZ` of this quaternion and store them in the 
        provided parameter `eulerAngles`.
        
        The Euler angles are always returned as the angle around X in the Vector3f.x field, the angle around Y in the Vector3f.y
        field and the angle around Z in the Vector3f.z field of the supplied vector.

        Arguments
        - eulerAngles: will hold the euler angles in radians

        Returns
        - the passed in vector
        """
        ...


    def lengthSquared(self) -> float:
        """
        Return the square of the length of this quaternion.

        Returns
        - the length
        """
        ...


    def slerp(self, target: "Quaternionfc", alpha: float, dest: "Quaternionf") -> "Quaternionf":
        """
        Interpolate between `this` .normalize(Quaternionf) unit quaternion and the specified
        `target` .normalize(Quaternionf) unit quaternion using spherical linear interpolation using the specified interpolation factor `alpha`,
        and store the result in `dest`.
        
        This method resorts to non-spherical linear interpolation when the absolute dot product of `this` and `target` is
        below `1E-6f`.
        
        Reference: <a href="http://fabiensanglard.net/doom3_documentation/37725-293747_293747.pdf">http://fabiensanglard.net</a>

        Arguments
        - target: the target of the interpolation, which should be reached with `alpha = 1.0`
        - alpha: the interpolation factor, within `[0..1]`
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def scale(self, factor: float, dest: "Quaternionf") -> "Quaternionf":
        """
        Apply scaling to this quaternion, which results in any vector transformed by the quaternion to change
        its length by the given `factor`, and store the result in `dest`.

        Arguments
        - factor: the scaling factor
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def integrate(self, dt: float, vx: float, vy: float, vz: float, dest: "Quaternionf") -> "Quaternionf":
        """
        Integrate the rotation given by the angular velocity `(vx, vy, vz)` around the x, y and z axis, respectively,
        with respect to the given elapsed time delta `dt` and add the differentiate rotation to the rotation represented by this quaternion
        and store the result into `dest`.
        
        This method pre-multiplies the rotation given by `dt` and `(vx, vy, vz)` by `this`, so
        the angular velocities are always relative to the local coordinate system of the rotation represented by `this` quaternion.
        
        This method is equivalent to calling: `rotateLocal(dt * vx, dt * vy, dt * vz, dest)`
        
        Reference: <a href="http://physicsforgames.blogspot.de/2010/02/quaternions.html">http://physicsforgames.blogspot.de/</a>

        Arguments
        - dt: the delta time
        - vx: the angular velocity around the x axis
        - vy: the angular velocity around the y axis
        - vz: the angular velocity around the z axis
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def nlerp(self, q: "Quaternionfc", factor: float, dest: "Quaternionf") -> "Quaternionf":
        """
        Compute a linear (non-spherical) interpolation of `this` and the given quaternion `q`
        and store the result in `dest`.
        
        Reference: <a href="http://fabiensanglard.net/doom3_documentation/37725-293747_293747.pdf">http://fabiensanglard.net</a>

        Arguments
        - q: the other quaternion
        - factor: the interpolation factor. It is between 0.0 and 1.0
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def nlerpIterative(self, q: "Quaternionfc", alpha: float, dotThreshold: float, dest: "Quaternionf") -> "Quaternionf":
        """
        Compute linear (non-spherical) interpolations of `this` and the given quaternion `q`
        iteratively and store the result in `dest`.
        
        This method performs a series of small-step nlerp interpolations to avoid doing a costly spherical linear interpolation, like
        .slerp(Quaternionfc, float, Quaternionf) slerp,
        by subdividing the rotation arc between `this` and `q` via non-spherical linear interpolations as long as
        the absolute dot product of `this` and `q` is greater than the given `dotThreshold` parameter.
        
        Thanks to `@theagentd` at <a href="http://www.java-gaming.org/">http://www.java-gaming.org/</a> for providing the code.

        Arguments
        - q: the other quaternion
        - alpha: the interpolation factor, between 0.0 and 1.0
        - dotThreshold: the threshold for the dot product of `this` and `q` above which this method performs another iteration
                 of a small-step linear interpolation
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def lookAlong(self, dir: "Vector3fc", up: "Vector3fc", dest: "Quaternionf") -> "Quaternionf":
        """
        Apply a rotation to this quaternion that maps the given direction to the positive Z axis, and store the result in `dest`.
        
        Because there are multiple possibilities for such a rotation, this method will choose the one that ensures the given up direction to remain
        parallel to the plane spanned by the `up` and `dir` vectors. 
        
        If `Q` is `this` quaternion and `R` the quaternion representing the 
        specified rotation, then the new quaternion will be `Q * R`. So when transforming a
        vector `v` with the new quaternion by using `Q * R * v`, the
        rotation added by this method will be applied first!
        
        Reference: <a href="http://answers.unity3d.com/questions/467614/what-is-the-source-code-of-quaternionlookrotation.html">http://answers.unity3d.com</a>

        Arguments
        - dir: the direction to map to the positive Z axis
        - up: the vector which will be mapped to a vector parallel to the plane 
                     spanned by the given `dir` and `up`
        - dest: will hold the result

        Returns
        - dest

        See
        - .lookAlong(float, float, float, float, float, float, Quaternionf)
        """
        ...


    def lookAlong(self, dirX: float, dirY: float, dirZ: float, upX: float, upY: float, upZ: float, dest: "Quaternionf") -> "Quaternionf":
        """
        Apply a rotation to this quaternion that maps the given direction to the positive Z axis, and store the result in `dest`.
        
        Because there are multiple possibilities for such a rotation, this method will choose the one that ensures the given up direction to remain
        parallel to the plane spanned by the `up` and `dir` vectors. 
        
        If `Q` is `this` quaternion and `R` the quaternion representing the 
        specified rotation, then the new quaternion will be `Q * R`. So when transforming a
        vector `v` with the new quaternion by using `Q * R * v`, the
        rotation added by this method will be applied first!
        
        Reference: <a href="http://answers.unity3d.com/questions/467614/what-is-the-source-code-of-quaternionlookrotation.html">http://answers.unity3d.com</a>

        Arguments
        - dirX: the x-coordinate of the direction to look along
        - dirY: the y-coordinate of the direction to look along
        - dirZ: the z-coordinate of the direction to look along
        - upX: the x-coordinate of the up vector
        - upY: the y-coordinate of the up vector
        - upZ: the z-coordinate of the up vector
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def rotateTo(self, fromDirX: float, fromDirY: float, fromDirZ: float, toDirX: float, toDirY: float, toDirZ: float, dest: "Quaternionf") -> "Quaternionf":
        """
        Apply a rotation to `this` that rotates the `fromDir` vector to point along `toDir` and
        store the result in `dest`.
        
        Since there can be multiple possible rotations, this method chooses the one with the shortest arc.
        
        If `Q` is `this` quaternion and `R` the quaternion representing the 
        specified rotation, then the new quaternion will be `Q * R`. So when transforming a
        vector `v` with the new quaternion by using `Q * R * v`, the
        rotation added by this method will be applied first!
        
        Reference: <a href="http://stackoverflow.com/questions/1171849/finding-quaternion-representing-the-rotation-from-one-vector-to-another#answer-1171995">stackoverflow.com</a>

        Arguments
        - fromDirX: the x-coordinate of the direction to rotate into the destination direction
        - fromDirY: the y-coordinate of the direction to rotate into the destination direction
        - fromDirZ: the z-coordinate of the direction to rotate into the destination direction
        - toDirX: the x-coordinate of the direction to rotate to
        - toDirY: the y-coordinate of the direction to rotate to
        - toDirZ: the z-coordinate of the direction to rotate to
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def rotateTo(self, fromDir: "Vector3fc", toDir: "Vector3fc", dest: "Quaternionf") -> "Quaternionf":
        """
        Apply a rotation to `this` that rotates the `fromDir` vector to point along `toDir` and
        store the result in `dest`.
        
        Because there can be multiple possible rotations, this method chooses the one with the shortest arc.
        
        If `Q` is `this` quaternion and `R` the quaternion representing the 
        specified rotation, then the new quaternion will be `Q * R`. So when transforming a
        vector `v` with the new quaternion by using `Q * R * v`, the
        rotation added by this method will be applied first!

        Arguments
        - fromDir: the starting direction
        - toDir: the destination direction
        - dest: will hold the result

        Returns
        - dest

        See
        - .rotateTo(float, float, float, float, float, float, Quaternionf)
        """
        ...


    def rotateX(self, angle: float, dest: "Quaternionf") -> "Quaternionf":
        """
        Apply a rotation to `this` quaternion rotating the given radians about the x axis
        and store the result in `dest`.
        
        If `Q` is `this` quaternion and `R` the quaternion representing the 
        specified rotation, then the new quaternion will be `Q * R`. So when transforming a
        vector `v` with the new quaternion by using `Q * R * v`, the
        rotation added by this method will be applied first!

        Arguments
        - angle: the angle in radians to rotate about the x axis
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def rotateY(self, angle: float, dest: "Quaternionf") -> "Quaternionf":
        """
        Apply a rotation to `this` quaternion rotating the given radians about the y axis
        and store the result in `dest`.
        
        If `Q` is `this` quaternion and `R` the quaternion representing the 
        specified rotation, then the new quaternion will be `Q * R`. So when transforming a
        vector `v` with the new quaternion by using `Q * R * v`, the
        rotation added by this method will be applied first!

        Arguments
        - angle: the angle in radians to rotate about the y axis
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def rotateZ(self, angle: float, dest: "Quaternionf") -> "Quaternionf":
        """
        Apply a rotation to `this` quaternion rotating the given radians about the z axis
        and store the result in `dest`.
        
        If `Q` is `this` quaternion and `R` the quaternion representing the 
        specified rotation, then the new quaternion will be `Q * R`. So when transforming a
        vector `v` with the new quaternion by using `Q * R * v`, the
        rotation added by this method will be applied first!

        Arguments
        - angle: the angle in radians to rotate about the z axis
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def rotateLocalX(self, angle: float, dest: "Quaternionf") -> "Quaternionf":
        """
        Apply a rotation to `this` quaternion rotating the given radians about the local x axis
        and store the result in `dest`.
        
        If `Q` is `this` quaternion and `R` the quaternion representing the 
        specified rotation, then the new quaternion will be `R * Q`. So when transforming a
        vector `v` with the new quaternion by using `R * Q * v`, the
        rotation represented by `this` will be applied first!

        Arguments
        - angle: the angle in radians to rotate about the local x axis
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def rotateLocalY(self, angle: float, dest: "Quaternionf") -> "Quaternionf":
        """
        Apply a rotation to `this` quaternion rotating the given radians about the local y axis
        and store the result in `dest`.
        
        If `Q` is `this` quaternion and `R` the quaternion representing the 
        specified rotation, then the new quaternion will be `R * Q`. So when transforming a
        vector `v` with the new quaternion by using `R * Q * v`, the
        rotation represented by `this` will be applied first!

        Arguments
        - angle: the angle in radians to rotate about the local y axis
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def rotateLocalZ(self, angle: float, dest: "Quaternionf") -> "Quaternionf":
        """
        Apply a rotation to `this` quaternion rotating the given radians about the local z axis
        and store the result in `dest`.
        
        If `Q` is `this` quaternion and `R` the quaternion representing the 
        specified rotation, then the new quaternion will be `R * Q`. So when transforming a
        vector `v` with the new quaternion by using `R * Q * v`, the
        rotation represented by `this` will be applied first!

        Arguments
        - angle: the angle in radians to rotate about the local z axis
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def rotateAxis(self, angle: float, axisX: float, axisY: float, axisZ: float, dest: "Quaternionf") -> "Quaternionf":
        """
        Apply a rotation to `this` quaternion rotating the given radians about the specified axis
        and store the result in `dest`.
        
        If `Q` is `this` quaternion and `R` the quaternion representing the 
        specified rotation, then the new quaternion will be `Q * R`. So when transforming a
        vector `v` with the new quaternion by using `Q * R * v`, the
        rotation added by this method will be applied first!

        Arguments
        - angle: the angle in radians to rotate about the specified axis
        - axisX: the x coordinate of the rotation axis
        - axisY: the y coordinate of the rotation axis
        - axisZ: the z coordinate of the rotation axis
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def rotateAxis(self, angle: float, axis: "Vector3fc", dest: "Quaternionf") -> "Quaternionf":
        """
        Apply a rotation to `this` quaternion rotating the given radians about the specified axis
        and store the result in `dest`.
        
        If `Q` is `this` quaternion and `R` the quaternion representing the 
        specified rotation, then the new quaternion will be `Q * R`. So when transforming a
        vector `v` with the new quaternion by using `Q * R * v`, the
        rotation added by this method will be applied first!

        Arguments
        - angle: the angle in radians to rotate about the specified axis
        - axis: the rotation axis
        - dest: will hold the result

        Returns
        - dest

        See
        - .rotateAxis(float, float, float, float, Quaternionf)
        """
        ...


    def difference(self, other: "Quaternionfc", dest: "Quaternionf") -> "Quaternionf":
        """
        Compute the difference between `this` and the `other` quaternion
        and store the result in `dest`.
        
        The difference is the rotation that has to be applied to get from
        `this` rotation to `other`. If `T` is `this`, `Q`
        is `other` and `D` is the computed difference, then the following equation holds:
        
        `T * D = Q`
        
        It is defined as: `D = T^-1 * Q`, where `T^-1` denotes the .invert(Quaternionf) inverse of `T`.

        Arguments
        - other: the other quaternion
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def positiveX(self, dir: "Vector3f") -> "Vector3f":
        """
        Obtain the direction of `+X` before the rotation transformation represented by `this` quaternion is applied.
        
        This method is equivalent to the following code:
        ```
        Quaternionf inv = new Quaternionf(this).invert();
        inv.transform(dir.set(1, 0, 0));
        ```

        Arguments
        - dir: will hold the direction of `+X`

        Returns
        - dir
        """
        ...


    def normalizedPositiveX(self, dir: "Vector3f") -> "Vector3f":
        """
        Obtain the direction of `+X` before the rotation transformation represented by `this` *normalized* quaternion is applied.
        The quaternion *must* be .normalize(Quaternionf) normalized for this method to work.
        
        This method is equivalent to the following code:
        ```
        Quaternionf inv = new Quaternionf(this).conjugate();
        inv.transform(dir.set(1, 0, 0));
        ```

        Arguments
        - dir: will hold the direction of `+X`

        Returns
        - dir
        """
        ...


    def positiveY(self, dir: "Vector3f") -> "Vector3f":
        """
        Obtain the direction of `+Y` before the rotation transformation represented by `this` quaternion is applied.
        
        This method is equivalent to the following code:
        ```
        Quaternionf inv = new Quaternionf(this).invert();
        inv.transform(dir.set(0, 1, 0));
        ```

        Arguments
        - dir: will hold the direction of `+Y`

        Returns
        - dir
        """
        ...


    def normalizedPositiveY(self, dir: "Vector3f") -> "Vector3f":
        """
        Obtain the direction of `+Y` before the rotation transformation represented by `this` *normalized* quaternion is applied.
        The quaternion *must* be .normalize(Quaternionf) normalized for this method to work.
        
        This method is equivalent to the following code:
        ```
        Quaternionf inv = new Quaternionf(this).conjugate();
        inv.transform(dir.set(0, 1, 0));
        ```

        Arguments
        - dir: will hold the direction of `+Y`

        Returns
        - dir
        """
        ...


    def positiveZ(self, dir: "Vector3f") -> "Vector3f":
        """
        Obtain the direction of `+Z` before the rotation transformation represented by `this` quaternion is applied.
        
        This method is equivalent to the following code:
        ```
        Quaternionf inv = new Quaternionf(this).invert();
        inv.transform(dir.set(0, 0, 1));
        ```

        Arguments
        - dir: will hold the direction of `+Z`

        Returns
        - dir
        """
        ...


    def normalizedPositiveZ(self, dir: "Vector3f") -> "Vector3f":
        """
        Obtain the direction of `+Z` before the rotation transformation represented by `this` *normalized* quaternion is applied.
        The quaternion *must* be .normalize(Quaternionf) normalized for this method to work.
        
        This method is equivalent to the following code:
        ```
        Quaternionf inv = new Quaternionf(this).conjugate();
        inv.transform(dir.set(0, 0, 1));
        ```

        Arguments
        - dir: will hold the direction of `+Z`

        Returns
        - dir
        """
        ...


    def conjugateBy(self, q: "Quaternionfc", dest: "Quaternionf") -> "Quaternionf":
        """
        Conjugate `this` by the given quaternion `q` by computing `q * this * q^-1`
        and store the result into `dest`.

        Arguments
        - q: the Quaternionfc to conjugate `this` by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def isFinite(self) -> bool:
        """
        Determine whether all components are finite floating-point values, that
        is, they are not Float.isNaN() NaN and not
        Float.isInfinite() infinity.

        Returns
        - `True` if all components are finite floating-point values;
                `False` otherwise
        """
        ...


    def equals(self, q: "Quaternionfc", delta: float) -> bool:
        """
             Compare the quaternion components of `this` quaternion with the given quaternion using the given `delta`
        and return whether all of them are equal within a maximum difference of `delta`.
        
        Please note that this method is not used by any data structure such as ArrayList HashSet or HashMap
        and their operations, such as ArrayList.contains(Object) or HashSet.remove(Object), since those
        data structures only use the Object.equals(Object) and Object.hashCode() methods.

        Arguments
        - q: the other quaternion
        - delta: the allowed maximum difference

        Returns
        - `True` whether all of the quaternion components are equal; `False` otherwise
        """
        ...


    def equals(self, x: float, y: float, z: float, w: float) -> bool:
        """
        Arguments
        - x: the x component to compare to
        - y: the y component to compare to
        - z: the z component to compare to
        - w: the w component to compare to

        Returns
        - `True` if all the quaternion components are equal
        """
        ...



from .LongStream import LongStream
from .IntStream import IntStream
from .DoubleStream import DoubleStream
from .Collector import Collector
from .Stream import Stream
from .BaseStream import BaseStream
from .StreamSupport import StreamSupport
from .Collectors import Collectors